@@ -478,7 +478,7 @@ realtype  S,TempI,Fii,TempJ,Fij, BB1,BB2;
                   (StepSize[i]*StepSize[i]);
       X[i]      = TempI+StepSize[i];
       if (i<N)
-        for (j=i+1;j<=N;j++)
+        for (j=i+1;j<=N;j++){
           if (!Freese[j])  {
             TempJ = X[j];
             X[j] += StepSize[j];
@@ -487,9 +487,11 @@ realtype  S,TempI,Fii,TempJ,Fij, BB1,BB2;
             Hsn[i][j] = (( Fc -FNeighbor[i] ) +
                          ( Fij-FNeighbor[j] )) /
                         (StepSize[i]*StepSize[j]);
-            X[j]      = TempJ;
-          } else
+            X[j] = TempJ;
+          } else {
             Hsn[i][j] = 0.0;
+	  }
+	}
       X[i] = TempI;
     } else
       for (j=i;j<=N;j++)
@@ -620,9 +622,10 @@ realtype Alpha, NewtLn, S, InitSp, RelLng, MinLam;
 realtype Lambda,LamTem, LamPre, FplPre, A, B;
 realtype Disc,  B1, B2, Lambda0;
 
+
+  LamPre = FplPre = 0.0; // to avoid them being undefined when
+  FplPre = 1.0;		 // computing B1 and B2
   MaxTkn  = False;
- LamPre = FplPre = 0.0;  // to avoid them being undefined when
-                         // computing B1 and B2
   RetCode = 2;
   Alpha   = 1.0e-4;
   NewtLn  = 0.0;
@@ -695,7 +698,7 @@ realtype Disc,  B1, B2, Lambda0;
            ( Lambda - LamPre );
       Disc = B*B - 3.0*A*InitSp;
       if (A==0.0)  LamTem = -InitSp/(2.0*B);
-             else  LamTem = (-B+sqrt(Disc))/(3.0*A);
+             else  LamTem = (-B+sqrt(RMax(Disc,0.0)))/(3.0*A);
       B1 = 0.5*Lambda;
       if (B1<LamTem)  LamTem = B1;
       LamPre = Lambda;
