@@ -64,7 +64,14 @@
 #endif
 
 
-#ifdef  _WIN32
+// _WIN32_NEWLINE should be raised when compilinig on Windows in
+// order to enforce Windows' line endings when writing text in
+// files opened for *binary* output. Otherwise, writing text lines
+// in binary files will results in UNIX line endings. Line endings
+// in files, opened for output in text mode, will be always
+// platform-specific.
+
+#ifdef  _WIN32_NEWLINE
 //  for DOS/WINDOWS machines:
 #define  NEWLINE  "\r\n"
 #else
@@ -186,7 +193,7 @@ CFile::~CFile()  {
 
 void  CFile::FreeBuffer ()  {
   if (IOBuf)  {
-    delete[] IOBuf;
+    if (!memIO)  delete[] IOBuf;
     IOBuf = NULL;
   }
   if (FName)  {
@@ -237,7 +244,7 @@ void  CFile::assign ( word poolSize, word sizeInc, pstr filePool )  {
   shut      ();
   FreeBuffer();
 
-  IOBuf    = filePool;
+  IOBuf    = (pstr)filePool;
   BufLen   = poolSize;
   FLength  = poolSize;
   BufInc   = sizeInc;
@@ -562,6 +569,7 @@ Boolean  CFile::rename ( cpstr NewFileName )  {
 }
 
 long  CFile::Position()  {
+// do not use on text files
 
   if (memIO)  return BufCnt;
 
@@ -571,6 +579,7 @@ long  CFile::Position()  {
 }
 
 Boolean  CFile::seek ( long Position )  {
+// do not use on text files
   if (memIO)  {
     if (Position<=(long)BufLen)  {
       BufCnt    = Position;
@@ -695,7 +704,7 @@ Boolean  HSuccess = IOSuccess;
                (LCnt<MaxLen-1)       &&
                (IOBuf[BufCnt]!='\r') &&
                (IOBuf[BufCnt]!='\n') )
-        Line[LCnt++] = IOBuf[BufCnt++];
+          Line[LCnt++] = IOBuf[BufCnt++];
         if (BufCnt>=BufLen)  {
           HSuccess  = IOSuccess;
           BufLen    = ReadFile ( IOBuf,Buf_Size );
@@ -981,7 +990,7 @@ byte       sl;
 word  CFile::ReadFile ( void * Buffer, word Count )  {
 word  Cnt;
   if (memIO)  {
-    Cnt       = WMax(Count,FLength-BufCnt);
+    Cnt       = WMin(Count,FLength-BufCnt);
     if (Cnt>0)  {
       memcpy ( Buffer,&(IOBuf[BufCnt]),Cnt );
       BufCnt += Cnt;
