@@ -50,6 +50,7 @@
 #include "mattype_.h"
 #endif
 
+#include <limits>
 #include <stdio.h>
 
 // -------------------------------------------------------
@@ -612,20 +613,26 @@ void FreeMatrix3Memory  ( psmatrix3 & A, word N,      word M,
 
 realtype  MachinEps ()  {
 //  A1.3.1   :  Calculation of the machine's epsilon
+/*
 realtype  rMachEps = 1.0;
   do
     rMachEps /= 2.0;
   while ((1.0+rMachEps)!=1.0);
   return  2.0*rMachEps;
+*/
+  return std::numeric_limits<realtype>::epsilon();
 }
 
 realtype  floatMachinEps()  {
 //  A1.3.1   :  Calculation of the machine's epsilon
+/*
 float fMachEps = 1.0;
   do
     fMachEps /= 2.0;
   while (float(1.0+fMachEps)!=1.0);
   return  2.0*fMachEps;
+*/
+  return std::numeric_limits<float>::epsilon();
 }
 
 realtype  frac ( realtype R )  {
@@ -1403,7 +1410,8 @@ int i,k;
 # define _nfPowers  255
 # define _nfPower0  127
 # define _nfPower8  135
-# define _nfPower4  131
+//# define _nfPower4  131
+# define _nfPower4  130
 #else
 # define _nfPowers  31
 # define _nfPower0  15
@@ -1414,6 +1422,7 @@ int i,k;
 static realtype _fpower[_nfPowers+1];
 static realtype _fpower8;
 static realtype _fpower4;
+static Boolean  _old_float_unibin;
 
 Boolean InitFPowers()  {
 int i;
@@ -1425,10 +1434,27 @@ int i;
   _fpower[_nfPowers] = fMaxReal;
   _fpower8 = _fpower[_nfPower8];
   _fpower4 = _fpower[_nfPower4];
+  _old_float_unibin = False;
   return True;
 }
 
+void __modify4()  {
+  _fpower4 = _fpower[_nfPower4-1];
+}
 
+void  set_new_float_unibin()  {
+  _old_float_unibin = False;
+}
+
+Boolean is_new_float_unibin()  {
+  return !_old_float_unibin;
+}
+
+void  set_old_float_unibin()  {
+  _old_float_unibin = True;
+}
+
+/*
 void  int2UniBin ( int I, intUniBin iUB )  {
 int j,n,sh;
   sh = 8*(sizeof(intUniBin)-1);
@@ -1438,6 +1464,17 @@ int j,n,sh;
     sh -= 8;
   }
 }
+*/
+
+void  int2UniBin ( int I, intUniBin iUB )  {
+int  n;
+word j;
+  n = I;
+  for (j=0;j<sizeof(intUniBin);j++)  {
+    iUB[j] = byte(n & 0xFF);
+    n >>= 8;
+  }
+}
 
 void  short2UniBin ( short S, shortUniBin sUB )  {
 int   j,sh;
@@ -1522,6 +1559,10 @@ realtype Q,L;
 
 }
 
+/*
+#undef _new_float_unibin
+
+#ifdef _new_float_unibin
 
 void  float2UniBin ( realtype R, floatUniBin fUB )  {
 int      k1,k2,k;
@@ -1548,6 +1589,149 @@ realtype Q,L;
 
 }
 
+#else
+
+void  float2UniBin ( realtype R, floatUniBin fUB )  {
+int      k1,k2,k;
+realtype Q,L;
+  if (R>=0)  Q = R;
+       else  Q = -R;
+  k1 = 0;
+  k2 = _nfPowers;
+  do {
+    k = (k1+k2)/2;
+    if (Q>=_fpower[k])  k1 = k;
+                  else  k2 = k;
+  } while (k2>k1+1);
+  if (Q<=_fpower[0])  k2 = 0;
+  Q = (Q/_fpower[k2])*_fpower8;
+  fUB[0] = byte(k2);
+  for (k=sizeof(realUniBin)-1;k>0;k--)  {
+    L = floor(Q/_rfbase);
+    if (k<=sizeof(floatUniBin))
+      fUB[k] = byte(int(Q-L*_rfbase));
+    Q = L;
+  }
+  if (R<0)  fUB[1] |= _fsign;
+
+}
+
+#endif
+*/
+
+void  float2UniBin ( realtype R, floatUniBin fUB )  {
+int      k1,k2,k;
+realtype Q,L;
+
+  if (R>=0)  Q = R;
+       else  Q = -R;
+  k1 = 0;
+  k2 = _nfPowers;
+  do {
+    k = (k1+k2)/2;
+    if (Q>=_fpower[k])  k1 = k;
+                  else  k2 = k;
+  } while (k2>k1+1);
+  if (Q<=_fpower[0])  k2 = 0;
+  fUB[0] = byte(k2);
+
+  if (_old_float_unibin)  {
+    // this is wrong but compatible with already existing files :(
+    // in the result, it gives errors in 6th digit at back conversion
+    Q = (Q/_fpower[k2])*_fpower8;
+    for (k=sizeof(realUniBin)-1;k>0;k--)  {
+      L = floor(Q/_rfbase);
+      if (k<=(int)sizeof(floatUniBin))
+        fUB[k] = byte(int(Q-L*_rfbase));
+      Q = L;
+    }
+  } else  {
+    // this is correct
+    Q = (Q/_fpower[k2])*_fpower4;
+    for (k=sizeof(floatUniBin)-1;k>0;k--)  {
+      L = floor(Q/_rfbase);
+      fUB[k] = byte(int(Q-L*_rfbase));
+      Q = L;
+    }
+  }
+
+//if (fUB[1] & _fsign)  printf ( " error!\n" );
+
+  if (R<0)  fUB[1] |= _fsign;
+
+}
+
+
+void  UniBin2float ( floatUniBin fUB, realtype & R )  {
+int j,s;
+
+  if (fUB[1] & _fsign)  {
+    s = 1;
+    fUB[1] &= _fsign1;
+  } else
+    s = 0;
+
+  R = int(fUB[1]);
+
+  if (_old_float_unibin)  {
+    // this is wrong and gives a conversion error in 6th digit :(
+    // we have to keep this for compatibility with already existing
+    // files
+    for (j=2;j<(int)sizeof(floatUniBin);j++)
+      R = R*_rfbase + int(fUB[j]);
+    for (j=sizeof(floatUniBin);j<(int)sizeof(realUniBin);j++)
+      R *= _rfbase;
+    R = (R/_fpower8)*_fpower[int(fUB[0])];
+  } else  {
+    // this is correct
+    for (j=2;j<(int)sizeof(floatUniBin);j++)
+      R = R*_rfbase + int(fUB[j]);
+    R = (R/_fpower4)*_fpower[int(fUB[0])];
+  }
+  if (s)  R = -R;
+}
+
+
+/* -------------------------------------------------------
+   This piece of code shows that float2Unibin - Unbin2float
+   pair does same-quality job as the native float - double
+   conversion:
+
+  InitMatType();
+  set_new_float_unibin();
+
+  floatUniBin      fUB;
+  realUniBin      rUB;
+  realtype         maxsh  = MaxShortReal/2.0; // max manageable /2!
+  float            maxshf = maxsh;
+  realtype         maxshr = maxshf;
+  realtype         maxsh1;
+
+  float2UniBin ( maxsh,fUB );
+  UniBin2float ( fUB,maxsh1 );
+
+  printf ( " float\n %10.3f\n %10.3f\n %10.3f\n %10.3f\n",
+           maxsh,maxsh1,maxshf,maxshr );
+
+  maxsh = MaxShortReal;
+  real2UniBin ( maxsh,rUB );
+  UniBin2real ( rUB,maxsh1 );
+
+  printf ( " real\n %10.3f\n %10.3f\n",maxsh,maxsh1 );
+
+---- RESULTS:
+
+ float
+ 170099999999999990938343446679146987520.000
+ 170099999948540854500627141228603899904.000
+ 170100000027769017014891478822147850240.000
+ 170100000027769017014891478822147850240.000
+ real
+ 340199999999999981876686893358293975040.000
+ 340199999999999981876686893358293975040.000
+
+-------------------------------------------------------------- */
+
 /*
 void  shortreal2UniBin ( shortreal R, shortrealUniBin srUB )  {
 int      k1,k2,k;
@@ -1602,7 +1786,7 @@ realtype Q,L;
 }
 */
 
-
+/*
 void  UniBin2int ( intUniBin iUB, int & I )  {
 int j,n,sh;
   sh = 8*sizeof(intUniBin);
@@ -1613,6 +1797,16 @@ int j,n,sh;
     I   = I | (n << sh);
   }
 }
+*/
+
+void  UniBin2int ( intUniBin iUB, int & I )  {
+int j;
+  I = 0x00;
+  for (j=sizeof(intUniBin)-1;j>=0;j--)  {
+    I <<= 8;
+    I |= int(iUB[j]);
+  }
+}
 
 void  UniBin2short ( shortUniBin sUB, short & S )  {
 int   j,sh;
@@ -1678,6 +1872,9 @@ int j,s;
   if (s)  R = -R;
 }
 
+/*
+#ifdef _new_float_unibin
+
 void  UniBin2float ( floatUniBin fUB, realtype & R )  {
 int j,s;
   if (fUB[1] & _fsign)  {
@@ -1692,6 +1889,31 @@ int j,s;
   if (s)  R = -R;
 }
 
+#else
+
+void  UniBin2float ( floatUniBin fUB, realtype & R )  {
+int j,s;
+  if (fUB[1] & _fsign)  {
+    s = 1;
+    fUB[1] &= _fsign1;
+  } else
+    s = 0;
+  R = int(fUB[1]);
+  for (j=2;j<sizeof(floatUniBin);j++)
+    R = R*_rfbase + int(fUB[j]);
+  for (j=sizeof(floatUniBin);j<sizeof(realUniBin);j++)
+    R *= _rfbase;
+  R = (R/_fpower8)*_fpower[int(fUB[0])];
+  if (s)  R = -R;
+}
+
+#endif
+*/
+
+
+
+
+
 /*
 void  UniBin2shortreal ( shortrealUniBin srUB, shortreal & R )  {
 int j,s;
@@ -1793,12 +2015,17 @@ int len;
   }
 }
 
-void mem_write ( Boolean  B, pstr S, int & l )  {
+void mem_write ( Boolean B, pstr S, int & l )  {
   if (B)  S[l++] = 'Y';
     else  S[l++] = 'N';
   S[l] = char(0);
 }
 
+void mem_write_byte ( byte B, pstr S, int & l )  {
+  S[l++] = char(B);
+  S[l]   = char(0);
+}
+
 
 void mem_read ( int & I, cpstr S, int & l )  {
 intUniBin iUB;
@@ -1863,8 +2090,11 @@ int len;
 }
 
 void mem_read ( Boolean & B, cpstr S, int & l )  {
-  B = (S[l]=='Y');
-  l++;
+  B = (S[l++]=='Y');
+}
+
+void mem_read_byte ( byte & B, cpstr S, int & l )  {
+  B = byte(S[l++]);
 }
 
 // -------------------------------------------------------
