@@ -5,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or
-//    modify it under the terms of the GNU Lesser General Public
-//    License version 3, modified in accordance with the provisions
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser
-//    General Public License along with this library. If not, copies
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    19.04.11   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -30,7 +30,7 @@
 //  **** Classes    :  CAlignment  ( alignment of character strings )
 //       ~~~~~~~~~~~~  CAlignment1 ( alignment of integer vectors   )
 //
-//  (C) E.Krissinel'  2000-2008
+//  (C) E.Krissinel'  2000-2011
 //
 //  =================================================================
 //
@@ -519,6 +519,50 @@ realtype CAlignment::Score ( char A, char B )  {
   return SNEq;
 }
 
+realtype CAlignment::GetSimilarity()  {
+realtype s,a;
+int      i,n;
+
+  s = 0.0;
+  a = 0.0;
+  n = IMin ( strlen(AlgnS),strlen(AlgnT) );
+
+  for (i=0;i<n;i++)
+    if ((AlgnS[i]!=Space) || (AlgnT[i]!=Space))  {
+      a += RMax ( Score(AlgnS[i],AlgnS[i]),Score(AlgnT[i],AlgnT[i]) );
+      s += Score ( AlgnS[i],AlgnT[i] );
+    }
+
+  if ((s>0.0) && (a>0.0))  return s/a;
+  return 0.0;
+
+}
+
+
+realtype CAlignment::GetSeqId()  {
+realtype s;
+int      i,n,ne,ns,nt;
+
+  ne = 0;
+  ns = 0;
+  nt = 0;
+  n  = IMin ( strlen(AlgnS),strlen(AlgnT) );
+
+  for (i=0;i<n;i++)  {
+    if (AlgnS[i]!=Space)  ns++;
+    if (AlgnT[i]!=Space)  {
+      nt++;
+      if (AlgnS[i]==AlgnT[i])
+        ne++;
+    }
+  }
+
+  s = IMin ( ns,nt );
+  if (s>0.0)  return ne/s;
+  return 0.0;
+
+}
+
 
 #define  WrapPeriod  61
 
@@ -1049,6 +1093,25 @@ realtype CAlignment1::Score ( int A, int B )  {
 }
 
 
+realtype CAlignment1::GetSimilarity()  {
+realtype s,a;
+int      i;
+
+  s = 0.0;
+  a = 0.0;
+
+  for (i=0;i<AlgnLen;i++)
+    if ((AlgnS[i]!=Space) || (AlgnT[i]!=Space))  {
+      a += RMax ( Score(AlgnS[i],AlgnS[i]),Score(AlgnT[i],AlgnT[i]) );
+      s += Score ( AlgnS[i],AlgnT[i] );
+    }
+
+  if ((s>0.0) && (a>0.0))  return s/a;
+  return 0.0;
+
+}
+
+
 void  CAlignment1::OutputResults ( RCFile  f, ivector S, int lenS,
                                    ivector T, int lenT )  {
 int   k,l,n;
