@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    14.06.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -38,7 +38,7 @@
 //                  CHetRec         ( HET    records                  )
 //                  CChain          ( MMDB chain class                )
 //
-//  Copyright (C) E. Krissinel 2000-2010
+//  Copyright (C) E. Krissinel 2000-2013
 //
 //  =================================================================
 //
@@ -547,6 +547,7 @@ int           RC;
   //  Determine the ChainID first and store it locally. It will
   // be used by CModel for generating chains and placing the
   // primary structure data BEFORE reading the coordinate section.
+
   F = Loop->GetString ( CIFTAG_NDB_PDB_CHAIN_ID,Signal,RC );
   if ((!RC) && F)  {
     strcpy_n0 ( chainID,F,sizeof(ChainID)-1 );
@@ -557,7 +558,8 @@ int           RC;
   CIFGetString ( resName,Loop,CIFTAG_MON_ID,Signal,sizeof(ResName),
                  pstr("UNK") );
 
-  if (CIFGetInteger(seqNum,Loop,CIFTAG_SEQ_NUM,Signal))  return;
+  CIFGetIntegerD ( seqNum,Loop,CIFTAG_SEQ_NUM );
+
   CIFGetString ( insCode,Loop,CIFTAG_NDB_PDB_INS_CODE,
                  Signal,sizeof(InsCode),pstr(" ") );
 
@@ -570,8 +572,9 @@ int           RC;
   CIFGetString ( dbRes,Loop,CIFTAG_DB_MON_ID,Signal,sizeof(ResName),
                  pstr("   ") );
 
-  if (CIFGetInteger1(dbSeq,Loop,CIFTAG_NDB_SEQ_DB_SEQ_NUM,Signal))
-    dbSeq = MinInt4;
+  CIFGetIntegerD ( dbSeq,Loop,CIFTAG_NDB_SEQ_DB_SEQ_NUM );
+//  if (CIFGetInteger1(dbSeq,Loop,CIFTAG_NDB_SEQ_DB_SEQ_NUM,Signal))
+//    dbSeq = MinInt4;
 
   F = Loop->GetString ( CIFTAG_DETAILS,Signal,RC );
   if ((!RC) && F)  {
@@ -1173,8 +1176,8 @@ int         RC;
   if (RC!=CIFRC_Ok)  {
     // the category was (re)created, provide tags
     Loop->AddLoopTag ( CIFTAG_ID              );
-    Loop->AddLoopTag ( CIFTAG_LABEL_ASYM_ID   );
-    Loop->AddLoopTag ( CIFTAG_LABEL_SEQ_ID    );
+    Loop->AddLoopTag ( CIFTAG_AUTH_ASYM_ID    );
+    Loop->AddLoopTag ( CIFTAG_AUTH_SEQ_ID     );
     Loop->AddLoopTag ( CIFTAG_INS_CODE        );
     Loop->AddLoopTag ( CIFTAG_NUMBER_ATOMS_NH );
     Loop->AddLoopTag ( CIFTAG_DETAILS         );
@@ -1210,10 +1213,10 @@ int           RC;
   //  Determine the ChainID first and store it locally. It will
   // be used by CModel for generating chains and placing the
   // primary structure data BEFORE reading the coordinate section.
-  F = Loop->GetString ( CIFTAG_LABEL_ASYM_ID,Signal,RC );
+  F = Loop->GetString ( CIFTAG_AUTH_ASYM_ID,Signal,RC );
   if ((!RC) && F)  {
     strcpy_n0 ( chainID,F,sizeof(ChainID)-1 );
-    Loop->DeleteField ( CIFTAG_LABEL_ASYM_ID,Signal );
+    Loop->DeleteField ( CIFTAG_AUTH_ASYM_ID,Signal );
   } else
     strcpy ( chainID,"" );
 
@@ -1221,7 +1224,7 @@ int           RC;
   CIFGetString ( hetID,Loop,CIFTAG_ID,Signal,sizeof(ResName),
                  pstr("UNK") );
 
-  if (CIFGetInteger(seqNum,Loop,CIFTAG_LABEL_SEQ_ID,Signal))  return;
+  if (CIFGetInteger(seqNum,Loop,CIFTAG_AUTH_SEQ_ID,Signal))  return;
 
   CIFGetString ( insCode,Loop,CIFTAG_INS_CODE,Signal,sizeof(InsCode),
                  pstr(" ") );
