@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    05.02.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -34,7 +34,7 @@
 //                  CTVect          ( translational vector class    )
 //                  CMMDBCryst      ( MMDB cryst. section class     )
 //
-//  (C) E. Krissinel 2000-2010
+//  (C) E. Krissinel 2000-2013
 //
 //  =================================================================
 //
@@ -561,8 +561,10 @@ int i,j,k;
   NCode  = 0;
 
   if (fullInit)  {
-    syminfo_lib  = NULL;
-    ignoreScalei = False;   // flag to ignore SCALEi cards
+    syminfo_lib   = NULL;
+    ignoreScalei  = False;  // flag to ignore SCALEi cards
+    processSG     = True;   // flag to process space group at file read
+    fixSpaceGroup = True;   // flag to fix space group at file read
   }
 
 }
@@ -716,8 +718,7 @@ char     c;
 
 }
 
-int  CMMDBCryst::ConvertPDBString ( pstr PDBString,
-                                    Boolean fixSpaceGroup ) {
+int  CMMDBCryst::ConvertPDBString ( pstr PDBString )  {
 // Interprets the ASCII PDB line and fills the corresponding fields.
 //   Returns zero if the line was converted, otherwise returns a
 // non-negative value of Error_XXXX.
@@ -746,7 +747,7 @@ PCTVect     tVect;
     CutSpaces ( spaceGroup,SCUTKEY_BEGEND );
     if (fixSpaceGroup)  FixSpaceGroup();
                   else  strcpy ( spaceGroupFix,spaceGroup );
-    if (spaceGroupFix[0])  {
+    if (spaceGroupFix[0] && processSG)  {
       if (SymOps.SetGroup(spaceGroupFix,syminfo_lib)==SYMOP_Ok)
         WhatIsSet |= CSET_SpaceGroup;
     }
@@ -895,7 +896,7 @@ char S[100];
 }
 
 
-int  CMMDBCryst::GetCIF ( PCMMCIFData CIF, Boolean fixSpaceGroup ) {
+int  CMMDBCryst::GetCIF ( PCMMCIFData CIF ) {
 PCMMCIFStruct Struct;
 int RC;
 
@@ -938,8 +939,10 @@ int RC;
         strcpy ( spaceGroup,"C 1 2 1" );
     }
     */
-    if (SymOps.SetGroup(spaceGroupFix,syminfo_lib)==SYMOP_Ok)
-      WhatIsSet |= CSET_SpaceGroup;
+    if (spaceGroupFix[0] && processSG)  {
+      if (SymOps.SetGroup(spaceGroupFix,syminfo_lib)==SYMOP_Ok)
+        WhatIsSet |= CSET_SpaceGroup;
+    }
   }
 
   if ((a*b*c*alpha*beta*gamma==0.0) ||
@@ -1710,26 +1713,26 @@ int      i,j,k;
     Tmp[1][2] = U[5];
     Tmp[2][1] = U[5];
 
-    for (i=0;i< 3;i++)
-      for (j=0;j< 3;j++)  {
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         A [j][i] = RFU[j][i];
         AT[i][j] = RFU[j][i];
       }
 
     //  TmpMat = Tmp*AT
-    for (i=0;i< 3;i++)
-      for (j=0;j< 3;j++)  {
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         BB = 0.0;
-        for (k=0;k< 3;k++)
+        for (k=0;k<3;k++)
           BB += Tmp[i][k]*AT[k][j];
         TmpMat[i][j] = BB;
       }
 
     //  Tmp = A*TmpMat
-    for (i=0;i< 3;i++)
-      for (j=0;j< 3;j++)  {
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         BB = 0.0;
-        for (k=0;k< 3;k++)
+        for (k=0;k<3;k++)
           BB += A[i][k]*TmpMat[k][j];
         Tmp[i][j] = BB;
       }
@@ -1830,8 +1833,8 @@ int RC,l;
       strcpy_ncss ( spaceGroup,spGroup,l );
       strcpy ( spaceGroupFix,spaceGroup );
       if (spaceGroup[0])  {
-	RC = SymOps.SetGroup ( spaceGroup,syminfo_lib );
-	//        RC = SymOps.SetGroup ( spGroup,syminfo_lib );
+        RC = SymOps.SetGroup ( spaceGroup,syminfo_lib );
+    //        RC = SymOps.SetGroup ( spGroup,syminfo_lib );
         //      strncpy ( spaceGroup,spGroup,l );
         //      spaceGroup[l] = char(0);
         if (RC==SYMOP_Ok)  WhatIsSet |= CSET_SpaceGroup;
@@ -2231,7 +2234,7 @@ PCMMDBCryst Cryst;
   if (f.reset()) {
     while (!f.FileEnd()) {
       f.ReadLine ( S,sizeof(S) );
-      Cryst->ConvertPDBString ( S,False );
+      Cryst->ConvertPDBString ( S );
     }
     f.shut();
   } else {
