@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    16.05.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -32,7 +32,7 @@
 //  **** Classes :  CMMDBFile  ( macromolecular data file class )
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2010
+//  (C) E. Krissinel 2000-2013
 //
 //  =================================================================
 //
@@ -96,7 +96,8 @@ void  CMMDBFile::InitMMDBFile()  {
   crRes   = NULL;
   lcount  = 0;
   strcpy ( S,"" );
-  Flags   = 0x00000000;           // no special effects
+//  Flags   = 0x00000000;           // no special effects
+  Flags   = MMDBF_IgnoreElement;  // done at request for default
   FType   = MMDB_FILE_Undefined;  // undefined file operation
   Exclude = True;
   ignoreRemarks     = False;  // used temporarily
@@ -171,24 +172,28 @@ void CMMDBFile::ResetManager() {
 
 void CMMDBFile::SetFlag ( word Flag )  {
   Flags |= Flag;
-  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
-  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
-  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
-  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
-  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
-  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID            ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement          ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge           ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors ) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch          ) != 0;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID       ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID   ) != 0;
+  Cryst.processSG        = (Flags & MMDBF_DoNotProcessSpaceGroup ) == 0;
+  Cryst.fixSpaceGroup    = (Flags & MMDBF_FixSpaceGroup          ) != 0;
 }
 
 void CMMDBFile::RemoveFlag ( word Flag )  {
   Flags &= ~Flag;
-  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
-  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
-  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
-  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
-  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
-  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID            ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement          ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge           ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors ) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch          ) != 0;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID       ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID   ) != 0;
+  Cryst.processSG        = (Flags & MMDBF_DoNotProcessSpaceGroup ) == 0;
+  Cryst.fixSpaceGroup    = (Flags & MMDBF_FixSpaceGroup          ) != 0;
 }
 
 
@@ -253,22 +258,14 @@ int  CMMDBFile::ReadPDBASCII ( RCFile f )  {
 PCContString ContString;
 word         cleanKey;
 int          RC,modNum;
-Boolean      fixSpaceGroup,fend;
+Boolean      fend;
 
   //  remove previous data
   ResetManager  ();
   FreeFileMemory();
 
   FType = MMDB_FILE_PDB;
-  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
-  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
-  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
-  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
-  ignoreRemarks          = False;
-  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
-  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
-  fixSpaceGroup          = (Flags & MMDBF_FixSpaceGroup         ) != 0;
+  SetFlag ( 0 );
 
   if (f.FileEnd())  return Error_EmptyFile;
 
@@ -340,6 +337,8 @@ Boolean      fixSpaceGroup,fend;
          strncmp(S,"ENDMDL",6))  {
     if (!strncmp(S,"LINK  ",6))
       crModel->ConvertPDBString ( S );
+    else if (!strncmp(S,"LINKR ",6))
+      crModel->ConvertPDBString ( S );
     else if (!strncmp(S,"CISPEP",6)) {
       GetInteger ( modNum,&(S[43]),3 );
       if (modNum<=0)  modNum = 1;
@@ -357,7 +356,7 @@ Boolean      fixSpaceGroup,fend;
 
   // read crystallographic information section
   do {
-    RC = Cryst.ConvertPDBString(S,fixSpaceGroup);
+    RC = Cryst.ConvertPDBString ( S );
     if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
       RC = 0;
     if (RC)  break;
@@ -370,7 +369,7 @@ Boolean      fixSpaceGroup,fend;
   } while (!fend);
 
   if (!RC)  {
-    RC = Cryst.ConvertPDBString(S,fixSpaceGroup);
+    RC = Cryst.ConvertPDBString ( S );
     if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
       RC = Error_WrongSection;
   }
@@ -456,21 +455,7 @@ pstr FName;
 
 int  CMMDBFile::ReadCIFASCII ( cpstr CIFFileName, byte gzipMode )  {
 CFile f;
-int   W;
-Boolean fixSpaceGroup;
-
-  //  remove previous data
-  ResetManager  ();
-  FreeFileMemory();
-  FType = MMDB_FILE_CIF;
-  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
-  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
-  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
-  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
-  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
-  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
-  fixSpaceGroup          = (Flags & MMDBF_FixSpaceGroup         ) != 0;
+int   rc;
 
   //  open the file as ASCII for reading
   //  opening it in pseudo-binary mode helps reading various
@@ -478,44 +463,62 @@ Boolean fixSpaceGroup;
   f.assign ( CIFFileName,False,False,gzipMode );
 
   if (f.reset(True)) {
-
-    CIFErrorLocation[0] = char(0);  // CIF reading phase
-
-    lcount = 0;  // line counter
-    S[0]   = char(0);
-
-    if (f.FileEnd())  {
-      f.shut();
-      return Error_EmptyFile;
-    }
-
-    if (!CIF)  CIF = new CMMCIFData();
-    CIF->SetStopOnWarning  ( True );
-    CIF->SetPrintWarnings  ( (Flags & MMDBF_PrintCIFWarnings)!=0 );
-    W = CIF->ReadMMCIFData ( f,S,lcount );
+    rc = ReadCIFASCII ( f );
     f.shut();
-    if (W)  {
-      if (W==CIFRC_NoDataLine)        return Error_NotACIFFile;
-      if (W & CIFW_UnrecognizedItems) return Error_UnrecognCIFItems;
-      if (W & CIFW_MissingField)      return Error_MissingCIFField;
-      if (W & CIFW_EmptyLoop)         return Error_EmptyCIFLoop;
-      if (W & CIFW_UnexpectedEOF)     return Error_UnexpEndOfCIF;
-      if (W & CIFW_LoopFieldMissing)  return Error_MissgCIFLoopField;
-      if (W & CIFW_NotAStructure)     return Error_NotACIFStructure;
-      if (W & CIFW_NotALoop)          return Error_NotACIFLoop;
-      return int(W);
-    }
-
-    return ReadFromCIF ( CIF,fixSpaceGroup );
-
   } else
-    return Error_CantOpenFile;
+    rc = Error_CantOpenFile;
+
+  return rc;
+
+}
+
+int  CMMDBFile::ReadCIFASCII ( RCFile f )  {
+int  W;
+
+  //  remove previous data
+  ResetManager  ();
+  FreeFileMemory();
+  FType = MMDB_FILE_CIF;
+
+  SetFlag ( 0 );
+
+  CIFErrorLocation[0] = char(0);  // CIF reading phase
+
+  lcount = 0;  // line counter
+  S[0]   = char(0);
+
+  if (f.FileEnd())
+    return Error_EmptyFile;
+
+  if (!CIF)  CIF = new CMMCIFData();
+  CIF->SetStopOnWarning  ( True );
+  CIF->SetPrintWarnings  ( (Flags & MMDBF_PrintCIFWarnings)!=0 );
+  W = CIF->ReadMMCIFData ( f,S,lcount );
+
+  if (W)  {
+    if (W==CIFRC_NoDataLine)        return Error_NotACIFFile;
+    if (W & CIFW_UnrecognizedItems) return Error_UnrecognCIFItems;
+    if (W & CIFW_MissingField)      return Error_MissingCIFField;
+    if (W & CIFW_EmptyLoop)         return Error_EmptyCIFLoop;
+    if (W & CIFW_UnexpectedEOF)     return Error_UnexpEndOfCIF;
+    if (W & CIFW_LoopFieldMissing)  return Error_MissgCIFLoopField;
+    if (W & CIFW_NotAStructure)     return Error_NotACIFStructure;
+    if (W & CIFW_NotALoop)          return Error_NotACIFLoop;
+    return int(W);
+  }
+
+  W = ReadFromCIF ( CIF );
+  if (CIF)  {
+    delete CIF;
+    CIF = NULL;
+  }
+
+  return W;
 
 }
 
 
-int CMMDBFile::ReadFromCIF ( PCMMCIFData CIFD,
-                             Boolean fixSpaceGroup )  {
+int CMMDBFile::ReadFromCIF ( PCMMCIFData CIFD )  {
 PCMMCIFLoop  Loop1,Loop2;
 pstr         F,FC;
 word         cleanKey;
@@ -536,7 +539,7 @@ int          RC,i,l,j,n,retc;
     return RC;
   }
 
-  RC = Cryst.GetCIF ( CIFD,fixSpaceGroup );
+  RC = Cryst.GetCIF ( CIFD );
   if (RC)  {
     CIFD->Optimize();
     return RC;
@@ -624,6 +627,35 @@ Boolean IBL;
 }
 
 
+int CMMDBFile::ReadCoorFile ( RCFile f )  {
+// auto format recognition
+int     kin;
+Boolean IBL;
+
+  kin = isMMDBBIN ( f );
+  f.reset ( True );
+  if (kin==Error_EmptyFile)
+              return Error_EmptyFile;
+  if (kin<0)  return Error_CantOpenFile;
+
+  if (kin==0) return  ReadMMDBF ( f );
+
+  IBL = ((Flags & MMDBF_IgnoreBlankLines)!=0);
+  kin = isPDB ( f,IBL );
+  f.reset ( True );
+  if (kin==0)
+    return ReadPDBASCII ( f );
+
+  kin = isCIF ( f );
+  f.reset ( True );
+  if (kin==0)
+    return ReadCIFASCII ( f );
+
+  return Error_ForeignFile;
+
+}
+
+
 word  CMMDBFile::PDBCleanup ( word CleanKey )  {
 //  cleans coordinate part to comply with PDB standards:
 //
@@ -675,8 +707,6 @@ pstr      chID;
 ChainID   chainID;
 Boolean   NewChain,Done,Solvent;
 
-  k = 0; // else may be used uninitialized
-
   RC = 0;
   if (nAtoms<=0)  return RC;
 
@@ -685,10 +715,10 @@ Boolean   NewChain,Done,Solvent;
       if (Atom[i])
         if (!Atom[i]->MakePDBAtomName())  RC |= PDBCLEAN_ATNAME;
 
+  k = -1;
 
   if (CleanKey & PDBCLEAN_TER)  {
     model    = -1;
-    k        = -1;
     crModel0 = crModel;
     for (i=0;i<nAtoms;i++)
       if (Atom[i])  {
@@ -972,20 +1002,23 @@ Boolean   NewChain,Done,Solvent;
         crModel0->nChains = k;
       }
     }
-    if (CleanKey & PDBCLEAN_CHAIN_ORDER)
-      for (i=0;i<nAtoms;i++)
+    if (CleanKey & PDBCLEAN_CHAIN_ORDER) {
+      for (i=0;i<nAtoms;i++) {
         if (Atom[i])  {
           crChain0 = Atom[i]->GetChain();
           crChain0->nWeights++;
           crChain0->Weight += Atom[i]->serNum;
         }
-    else
-      for (i=0;i<nAtoms;i++)
-        if (Atom[i])  {
-          crChain0 = Atom[i]->GetChain();
-          crChain0->nWeights++;
-          crChain0->Weight += Atom[i]->GetIndex();
-        }
+	else{
+	  for (i=0;i<nAtoms;i++)
+	    if (Atom[i])  {
+	      crChain0 = Atom[i]->GetChain();
+	      crChain0->nWeights++;
+	      crChain0->Weight += Atom[i]->GetIndex();
+	    }
+	}
+      }
+    }
     for (i=0;i<nModels;i++)  {
       crModel0 = Model[i];
       if (crModel0)  {
@@ -1405,7 +1438,7 @@ PCContString ContString;
   if (RC!=Error_WrongSection)  return RC;
 
   // belongs to the crystallographic information section?
-  RC = Cryst.ConvertPDBString ( S,False );
+  RC = Cryst.ConvertPDBString ( S );
   if (RC!=Error_WrongSection)  {
 //    if (RC==0)  Cryst.CalcCoordTransforms();
     return RC;
@@ -1570,7 +1603,7 @@ int  CMMDBFile::ReadPDBAtom ( cpstr L )  {
 // code.
 //   If L does not belong to the coordinate section,
 // Error_WrongSection is returned.
-int RC,index;
+int RC,index,i;
 
   if (!strncmp(L,"ATOM  ",6)) {
 
@@ -1612,10 +1645,15 @@ int RC,index;
 
     modelCnt++;
     RC = SwitchModel ( L );
-    if (!RC)  {
-      if (crModel->serNum!=modelCnt)
-        RC = Error_DuplicatedModel;
-    }
+    for (i=0;(i<nModels) && (!RC);i++)
+      if (Model[i] && (Model[i]!=crModel))  {
+        if (crModel->serNum==Model[i]->serNum)
+          RC = Error_DuplicatedModel;
+      }
+//    if (!RC)  {
+//      if (crModel->serNum!=modelCnt)
+//        RC = Error_DuplicatedModel;
+//    }
 
   } else if (!strncmp(L,"ENDMDL",6)) {
 
@@ -1731,11 +1769,12 @@ int i,kndex,RC;
   }
 
   if (kndex==0)  kndex = nAtoms+1;
-	
+
   if (!crModel)  SwitchModel ( 1 );
 
 
-  RC = AllocateAtom ( kndex,chainID,resName,seqNum,insCode,True );
+  RC = AllocateAtom ( kndex,chainID,chainID,resName,resName,
+                      seqNum,seqNum,1,insCode,True );
   if (!RC)
     Atom[kndex-1]->SetAtomName ( kndex,serNum,atomName,altLoc,
                                  segID,element );
@@ -1791,8 +1830,11 @@ int i,kndex,RC,sn;
   if (kndex==0)  kndex = nAtoms+1;
 
 
-  RC = AllocateAtom ( kndex,A->GetChainID(),A->GetResName(),
-                      A->GetSeqNum(),A->GetInsCode(),True );
+  RC = AllocateAtom ( kndex,A->GetChainID(),A->GetLabelAsymID(),
+                            A->GetResName(),A->GetLabelCompID(),
+                            A->GetSeqNum (),A->GetLabelSeqID (),
+                            A->GetLabelEntityID(),A->GetInsCode(),
+                            True );
 
   if (serNum<=0)  sn = kndex;
             else  sn = serNum;
@@ -1863,10 +1905,9 @@ int CMMDBFile::CheckInAtoms ( int index, // same meaning as above
 PPCAtom A1;
 int     i,j,k,k1,kndex;
 
-  A1 = (PCAtom *) NULL;  // else may be used uninitialized 
-
   if (!A)  return -1;
 
+  A1    = NULL;
   kndex = index;
 
   if (kndex<0)  {  // the new atoms are to be inserted
@@ -1877,16 +1918,12 @@ int     i,j,k,k1,kndex;
       ExpandAtomArray ( IMax(kndex,nAtoms)+1000+natms-AtmLen );
 
     if (kndex<nAtoms)
-      A1 = new PCAtom[natms];
-
-    // A1 may be undefined at this point, yet it is used in the
-    // next loop!
-
+    A1 = new PCAtom[natms];
     k = kndex-1;
     j = 0;
     for (i=0;i<natms;i++)
       if (A[i])  {
-        if (Atom[k])  A1[j++] = Atom[k]; 
+        if (Atom[k])  A1[j++] = Atom[k];
         Atom[k] = A[i];
         Atom[k]->index = k+1;
         k++;
@@ -2030,31 +2067,32 @@ InsCode  insCode;
     insCode[1] = char(0);
   }
 
-  return AllocateAtom ( index ,chainID,resName,
-                        seqNum,insCode,False );
+  return AllocateAtom ( index ,chainID,chainID,resName,resName,
+                        seqNum,seqNum,1,insCode,False );
 
 }
 
 int CMMDBFile::CheckAtomPlace ( int index, PCMMCIFLoop Loop )  {
 //   Version of CheckAtomPlace(..) for reading from CIF file.
-ResName  resName;
-int      seqNum,RC,k,nM;
-ChainID  chainID;
+ResName  resName,label_comp_id;
+int      seqNum ,label_seq_id,label_entity_id,RC,k,nM;
+ChainID  chainID,label_asym_id;
 InsCode  insCode;
 pstr     F;
 
   // Get the residue sequence number/insert code. They are
   // removed from the file after reading.
   k = index-1;
-  if (!CIFGetInteger1(seqNum,Loop,CIFTAG_LABEL_SEQ_ID,k))
-    CIFGetString  ( insCode,Loop,CIFTAG_NDB_INS_CODE,k,
+//  if (!CIFGetInteger1(seqNum,Loop,CIFTAG_LABEL_SEQ_ID,k))
+  if (!CIFGetInteger1(seqNum,Loop,CIFTAG_AUTH_SEQ_ID,k))
+    CIFGetString  ( insCode,Loop,CIFTAG_NDB_HELIX_CLASS_PDB,k,
                     sizeof(InsCode),pstr("") );
   else  {
     F = Loop->GetString ( CIFTAG_GROUP_PDB,k,RC );
     if ((!F) || (RC)) return  Error_CIF_EmptyRow;
     if (strcmp(F,"TER"))  {
       seqNum = MinInt4;  // only at reading CIF we allow this
-      CIFGetString ( insCode,Loop,CIFTAG_NDB_INS_CODE,k,
+      CIFGetString ( insCode,Loop,CIFTAG_NDB_HELIX_CLASS_PDB,k,
                      sizeof(InsCode),pstr("") );
     } else  { // we allow for empty TER card here
       seqNum     = 0;
@@ -2065,12 +2103,22 @@ pstr     F;
     }
   }
 
+  CIFGetInteger1 ( label_seq_id   ,Loop,CIFTAG_LABEL_SEQ_ID   ,k );
+  CIFGetInteger1 ( label_entity_id,Loop,CIFTAG_LABEL_ENTITY_ID,k );
+
   // get chain/residue ID
-  CIFGetString ( chainID,Loop,CIFTAG_LABEL_ASYM_ID,k,
+  CIFGetString ( chainID,Loop,CIFTAG_AUTH_ASYM_ID,k,
                  sizeof(ChainID),pstr("") );
-  CIFGetString ( resName,Loop,CIFTAG_LABEL_COMP_ID,k,
+  CIFGetString ( resName,Loop,CIFTAG_AUTH_COMP_ID,k,
                  sizeof(ResName),pstr("") );
 
+  CIFGetString ( label_asym_id,Loop,CIFTAG_LABEL_ASYM_ID,k,
+                 sizeof(ChainID),pstr("") );
+  CIFGetString ( label_comp_id,Loop,CIFTAG_LABEL_COMP_ID,k,
+                 sizeof(ResName),pstr("") );
+
+  if (!resName[0])  strcpy ( resName,label_comp_id );
+
   if (!CIFGetInteger1(nM,Loop,CIFTAG_PDBX_PDB_MODEL_NUM,k))  {
     if (crModel)  {
       if (nM!=crModel->serNum)  SwitchModel ( nM );
@@ -2078,16 +2126,21 @@ pstr     F;
       SwitchModel ( nM );
   }
 
-  return AllocateAtom ( index ,chainID,resName,
-                        seqNum,insCode,False );
+  return AllocateAtom ( index ,chainID,label_asym_id,resName,
+                        label_comp_id,seqNum,label_seq_id,
+                        label_entity_id,insCode,False );
 
 }
 
 
 int  CMMDBFile::AllocateAtom ( int           index,
                                const ChainID chainID,
+                               const ChainID label_asym_id,
                                const ResName resName,
+                               const ResName label_comp_id,
                                int           seqNum,
+                               int           label_seq_id,
+                               int           label_entity_id,
                                const InsCode insCode,
                                Boolean       Replace )  {
 
@@ -2150,6 +2203,11 @@ int  CMMDBFile::AllocateAtom ( int           index,
     if (!crRes)  return  Error_DuplicateSeqNum;
   }
 
+  strcpy ( crRes->label_asym_id,label_asym_id );
+  strcpy ( crRes->label_comp_id,label_comp_id );
+  crRes->label_seq_id    = label_seq_id;
+  crRes->label_entity_id = label_entity_id;
+
   // now check if there is place in the Atom array
   if (index>AtmLen)
     // there is no place, expand Atom by 1000 atom places at once
@@ -2216,8 +2274,8 @@ pstr FName;
 int  CMMDBFile::WritePDBASCII ( cpstr PDBFileName, byte gzipMode )  {
 CFile f;
 
-  //  opening it in pseudo-text mode ensures that the line terminators
-  //  will correspond to the system MMDB is running on
+  //  opening it in pseudo-text mode ensures that the line
+  //  endings will correspond to the system MMDB is running on
   f.assign ( PDBFileName,True,False,gzipMode );
   FType = MMDB_FILE_PDB;
 
@@ -2324,27 +2382,35 @@ pstr FName;
 }
 
 int  CMMDBFile::ReadMMDBF ( cpstr MMDBFileName, byte gzipMode )  {
-char  Label[100];
-byte  Version;
 CFile f;
+int   rc;
 
   f.assign ( MMDBFileName,False,True,gzipMode );
   FType = MMDB_FILE_Binary;
   if (f.reset(True))  {
-    f.ReadFile ( Label,sizeof(MMDBFLabel) );
-    if (strncmp(Label,MMDBFLabel,sizeof(MMDBFLabel)))  {
-      f.shut();
-      return Error_ForeignFile;
-    }
-    f.ReadByte ( &Version );
-    if (Version>Edition)  {
-      f.shut();
-      return Error_WrongEdition;
-    }
-    read ( f );
+    rc = ReadMMDBF ( f );
     f.shut();
   } else
-    return Error_CantOpenFile;
+    rc = Error_CantOpenFile;
+
+  return rc;
+
+}
+
+int  CMMDBFile::ReadMMDBF ( RCFile f )  {
+char  Label[100];
+byte  Version;
+
+  FType = MMDB_FILE_Binary;
+  f.ReadFile ( Label,sizeof(MMDBFLabel) );
+  if (strncmp(Label,MMDBFLabel,sizeof(MMDBFLabel)))
+    return Error_ForeignFile;
+
+  f.ReadByte ( &Version );
+  if (Version>Edition)
+    return Error_WrongEdition;
+
+  read ( f );
 
   return 0;
 
@@ -2639,6 +2705,20 @@ PCExpData  expData;
 }
 
 
+//  -------------------------------------------------------------
+
+int  CMMDBFile::GetNofMdlTypeRecs()  {
+  return Title.MdlType.Length();
+}
+
+pstr  CMMDBFile::GetMdlTypeRec ( int recNo )  {
+PCMdlType  mdlType;
+  mdlType = PCMdlType(Title.MdlType.GetContainerClass(recNo));
+  if (mdlType)  return mdlType->Line;
+  return NULL;
+}
+
+
 //  -------------------  Stream functions  ----------------------
 
 void  CMMDBFile::Copy ( PCMMDBFile MMDBFile )  {
@@ -2852,114 +2932,131 @@ byte Version;
 MakeStreamFunctions(CMMDBFile)
 
 
-
 int isMMDBBIN ( cpstr FName, byte gzipMode )  {
-char  Label[100];
-byte  Version;
 CFile f;
+int   rc;
 
   f.assign ( FName,False,True,gzipMode );
   if (f.reset(True))  {
-    if (f.FileEnd())  {
-      f.shut();
-      return Error_EmptyFile;
-    }
-    f.ReadFile ( Label,sizeof(MMDBFLabel) );
-    if (strncmp(Label,MMDBFLabel,sizeof(MMDBFLabel)))  {
-      f.shut();
-      return 1;
-    }
-    f.ReadByte ( &Version );
+    rc = isMMDBBIN ( f );
     f.shut();
-    if (Version>Edition)  return 2;
-                    else  return 0;
   } else
-    return -1;
+    rc = -1;
+
+  return rc;
+
+}
+
+int isMMDBBIN ( RCFile f )  {
+char  Label[100];
+byte  Version;
+
+  if (f.FileEnd())
+    return Error_EmptyFile;
+
+  f.ReadFile ( Label,sizeof(MMDBFLabel) );
+  if (strncmp(Label,MMDBFLabel,sizeof(MMDBFLabel)))
+    return 1;
+
+  f.ReadByte ( &Version );
+
+  if (Version>Edition)  return 2;
+                  else  return 0;
 
 }
 
 
 int isPDB ( cpstr FName, byte gzipMode, Boolean IgnoreBlankLines )  {
-CFile   f;
+CFile f;
+int   rc;
+
+  f.assign ( FName,False,False,gzipMode );
+  if (f.reset(True))  {
+    //  opening it in pseudo-binary mode helps reading various
+    //  line terminators for files coming from different platforms
+    rc = isPDB ( f,IgnoreBlankLines );
+    f.shut();
+  } else
+    rc = -1;
+
+  return rc;
+
+}
+
+int isPDB ( RCFile f, Boolean IgnoreBlankLines )  {
 char    S[256];
 int     i;
 Boolean Done;
 
-  //  opening it in pseudo-binary mode helps reading various
-  //  line terminators for files coming from different platforms
-  f.assign ( FName,False,False,gzipMode );
-  if (f.reset(True))  {
-    if (f.FileEnd())  {
-      f.shut();
-      return Error_EmptyFile;
+  if (f.FileEnd())
+    return Error_EmptyFile;
+
+  do {
+    Done = True;
+    f.ReadLine ( S,sizeof(S)-1 );
+    if (IgnoreBlankLines)  {
+      i = 0;
+      while (S[i] && (S[i]==' '))  i++;
+      if (!S[i])  Done = False;
     }
-    do {
-      Done = True;
-      f.ReadLine ( S,sizeof(S)-1 );
-      if (IgnoreBlankLines)  {
-        i = 0;
-        while (S[i] && (S[i]==' '))  i++;
-        if (!S[i])  Done = False;
-      }
-    } while ((!f.FileEnd()) && (!Done));
-    f.shut();
-    PadSpaces  ( S,80 );
-    if (!strncasecmp(S,"HEADER",6))  return 0;
-    if (!strncasecmp(S,"OBSLTE",6))  return 0;
-    if (!strncasecmp(S,"TITLE ",6))  return 0;
-    if (!strncasecmp(S,"CAVEAT",6))  return 0;
-    if (!strncasecmp(S,"COMPND",6))  return 0;
-    if (!strncasecmp(S,"SOURCE",6))  return 0;
-    if (!strncasecmp(S,"KEYWDS",6))  return 0;
-    if (!strncasecmp(S,"EXPDTA",6))  return 0;
-    if (!strncasecmp(S,"AUTHOR",6))  return 0;
-    if (!strncasecmp(S,"REVDAT",6))  return 0;
-    if (!strncasecmp(S,"SPRSDE",6))  return 0;
-    if (!strncasecmp(S,"JRNL  ",6))  return 0;
-    if (!strncasecmp(S,"REMARK",6))  return 0;
-    if (!strncasecmp(S,"DBREF ",6))  return 0;
-    if (!strncasecmp(S,"SEQADV",6))  return 0;
-    if (!strncasecmp(S,"SEQRES",6))  return 0;
-    if (!strncasecmp(S,"MODRES",6))  return 0;
-    if (!strncasecmp(S,"HET   ",6))  return 0;
-    if (!strncasecmp(S,"HETNAM",6))  return 0;
-    if (!strncasecmp(S,"HETSYN",6))  return 0;
-    if (!strncasecmp(S,"FORMUL",6))  return 0;
-    if (!strncasecmp(S,"HELIX ",6))  return 0;
-    if (!strncasecmp(S,"SHEET ",6))  return 0;
-    if (!strncasecmp(S,"TURN  ",6))  return 0;
-    if (!strncasecmp(S,"SSBOND",6))  return 0;
-    if (!strncasecmp(S,"LINK  ",6))  return 0;
-    if (!strncasecmp(S,"HYDBND",6))  return 0;
-    if (!strncasecmp(S,"SLTBRG",6))  return 0;
-    if (!strncasecmp(S,"CISPEP",6))  return 0;
-    if (!strncasecmp(S,"SITE  ",6))  return 0;
-    if (!strncasecmp(S,"CRYST1",6))  return 0;
-    if (!strncasecmp(S,"CRYST ",6))  return 0;
-    if (!strncasecmp(S,"ORIGX1",6))  return 0;
-    if (!strncasecmp(S,"ORIGX2",6))  return 0;
-    if (!strncasecmp(S,"ORIGX3",6))  return 0;
-    if (!strncasecmp(S,"SCALE1",6))  return 0;
-    if (!strncasecmp(S,"SCALE2",6))  return 0;
-    if (!strncasecmp(S,"SCALE3",6))  return 0;
-    if (!strncasecmp(S,"MTRIX1",6))  return 0;
-    if (!strncasecmp(S,"MTRIX2",6))  return 0;
-    if (!strncasecmp(S,"MTRIX3",6))  return 0;
-    if (!strncasecmp(S,"TVECT ",6))  return 0;
-    if (!strncasecmp(S,"MODEL ",6))  return 0;
-    if (!strncasecmp(S,"ATOM  ",6))  return 0;
-    if (!strncasecmp(S,"SIGATM",6))  return 0;
-    if (!strncasecmp(S,"ANISOU",6))  return 0;
-    if (!strncasecmp(S,"SIGUIJ",6))  return 0;
-    if (!strncasecmp(S,"TER   ",6))  return 0;
-    if (!strncasecmp(S,"HETATM",6))  return 0;
-    if (!strncasecmp(S,"ENDMDL",6))  return 0;
-    if (!strncasecmp(S,"CONECT",6))  return 0;
-    if (!strncasecmp(S,"MASTER",6))  return 0;
-    if (!strncasecmp(S,"END   ",6))  return 0;
-    if (!strncasecmp(S,"USER  ",6))  return 0;
-    return  1;
-  } else
-    return -1;
+  } while ((!f.FileEnd()) && (!Done));
+
+  PadSpaces  ( S,80 );
+  if (!strncasecmp(S,"HEADER",6))  return 0;
+  if (!strncasecmp(S,"OBSLTE",6))  return 0;
+  if (!strncasecmp(S,"TITLE ",6))  return 0;
+  if (!strncasecmp(S,"CAVEAT",6))  return 0;
+  if (!strncasecmp(S,"COMPND",6))  return 0;
+  if (!strncasecmp(S,"SOURCE",6))  return 0;
+  if (!strncasecmp(S,"KEYWDS",6))  return 0;
+  if (!strncasecmp(S,"EXPDTA",6))  return 0;
+  if (!strncasecmp(S,"AUTHOR",6))  return 0;
+  if (!strncasecmp(S,"REVDAT",6))  return 0;
+  if (!strncasecmp(S,"SPRSDE",6))  return 0;
+  if (!strncasecmp(S,"JRNL  ",6))  return 0;
+  if (!strncasecmp(S,"REMARK",6))  return 0;
+  if (!strncasecmp(S,"DBREF ",6))  return 0;
+  if (!strncasecmp(S,"SEQADV",6))  return 0;
+  if (!strncasecmp(S,"SEQRES",6))  return 0;
+  if (!strncasecmp(S,"MODRES",6))  return 0;
+  if (!strncasecmp(S,"HET   ",6))  return 0;
+  if (!strncasecmp(S,"HETNAM",6))  return 0;
+  if (!strncasecmp(S,"HETSYN",6))  return 0;
+  if (!strncasecmp(S,"FORMUL",6))  return 0;
+  if (!strncasecmp(S,"HELIX ",6))  return 0;
+  if (!strncasecmp(S,"SHEET ",6))  return 0;
+  if (!strncasecmp(S,"TURN  ",6))  return 0;
+  if (!strncasecmp(S,"SSBOND",6))  return 0;
+  if (!strncasecmp(S,"LINK  ",6))  return 0;
+  if (!strncasecmp(S,"HYDBND",6))  return 0;
+  if (!strncasecmp(S,"SLTBRG",6))  return 0;
+  if (!strncasecmp(S,"CISPEP",6))  return 0;
+  if (!strncasecmp(S,"SITE  ",6))  return 0;
+  if (!strncasecmp(S,"CRYST1",6))  return 0;
+  if (!strncasecmp(S,"CRYST ",6))  return 0;
+  if (!strncasecmp(S,"ORIGX1",6))  return 0;
+  if (!strncasecmp(S,"ORIGX2",6))  return 0;
+  if (!strncasecmp(S,"ORIGX3",6))  return 0;
+  if (!strncasecmp(S,"SCALE1",6))  return 0;
+  if (!strncasecmp(S,"SCALE2",6))  return 0;
+  if (!strncasecmp(S,"SCALE3",6))  return 0;
+  if (!strncasecmp(S,"MTRIX1",6))  return 0;
+  if (!strncasecmp(S,"MTRIX2",6))  return 0;
+  if (!strncasecmp(S,"MTRIX3",6))  return 0;
+  if (!strncasecmp(S,"TVECT ",6))  return 0;
+  if (!strncasecmp(S,"MODEL ",6))  return 0;
+  if (!strncasecmp(S,"ATOM  ",6))  return 0;
+  if (!strncasecmp(S,"SIGATM",6))  return 0;
+  if (!strncasecmp(S,"ANISOU",6))  return 0;
+  if (!strncasecmp(S,"SIGUIJ",6))  return 0;
+  if (!strncasecmp(S,"TER   ",6))  return 0;
+  if (!strncasecmp(S,"HETATM",6))  return 0;
+  if (!strncasecmp(S,"ENDMDL",6))  return 0;
+  if (!strncasecmp(S,"CONECT",6))  return 0;
+  if (!strncasecmp(S,"MASTER",6))  return 0;
+  if (!strncasecmp(S,"END   ",6))  return 0;
+  if (!strncasecmp(S,"USER  ",6))  return 0;
+
+  return  1;
 
 }
