@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    22.05.12   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //                  CMatch      ( match of structural graphs         )
 //                  CGraphMatch ( CSIA algorithms for graph matching )
 //
-//   (C) E. Krissinel 2000-2008
+//   (C) E. Krissinel 2000-2012
 //
 //  When used, please cite:
 //
@@ -262,6 +262,8 @@ int Version;
 }
 
 void  CVertex::mem_write ( pstr S, int & l )  {
+byte Version=2;
+  ::mem_write_byte ( Version,S,l );
   ::mem_write ( name    ,S,l );
   ::mem_write ( type    ,S,l );
   ::mem_write ( property,S,l );
@@ -271,6 +273,8 @@ void  CVertex::mem_write ( pstr S, int & l )  {
 }
 
 void  CVertex::mem_read ( cpstr S, int & l )  {
+byte Version;
+  ::mem_read_byte ( Version,S,l );
   ::mem_read ( name    ,S,l );
   ::mem_read ( type    ,S,l );
   ::mem_read ( property,S,l );
@@ -279,7 +283,7 @@ void  CVertex::mem_read ( cpstr S, int & l )  {
   ::mem_read ( type_ext,S,l );
 }
 
-MakeStreamFunctions(CVertex)
+MakeStreamFunctions(CVertex);
 
 
 
@@ -387,6 +391,8 @@ int Version;
 }
 
 void  CEdge::mem_write ( pstr S, int & l )  {
+byte Version=1;
+  ::mem_write_byte ( Version,S,l );
   ::mem_write ( v1      ,S,l );
   ::mem_write ( v2      ,S,l );
   ::mem_write ( type    ,S,l );
@@ -394,6 +400,8 @@ void  CEdge::mem_write ( pstr S, int & l )  {
 }
 
 void  CEdge::mem_read ( cpstr S, int & l )  {
+byte Version;
+  ::mem_read_byte ( Version,S,l );
   ::mem_read ( v1      ,S,l );
   ::mem_read ( v2      ,S,l );
   ::mem_read ( type    ,S,l );
@@ -401,7 +409,7 @@ void  CEdge::mem_read ( cpstr S, int & l )  {
 }
 
 
-MakeStreamFunctions(CEdge)
+MakeStreamFunctions(CEdge);
 
 
 
@@ -978,7 +986,8 @@ ivector v,vc;
 int     i,j,k,n,m,almask,vjtype, ctype,otype;
 Boolean noOxygens;
 
-  ctype = otype = 0; // else may be used uninitialized 
+  otype = 0;
+  ctype = 0;
 
   GetVectorMemory ( v ,nVertices,0 );
   GetVectorMemory ( vc,nVertices,1 );
@@ -1145,6 +1154,46 @@ int         i,j,n,nrings,nv;
 
 }
 
+void CGraph::markConnected ( int vno, int cno )  {
+int i;
+
+  Vertex[vno]->type_ext = cno;
+  for (i=0;i<nVertices;i++)
+    if (graph[vno+1][i+1] && (!Vertex[i]->type_ext))
+      markConnected ( i,cno );
+
+}
+
+
+int  CGraph::IdentifyConnectedComponents()  {
+// Returns the number of connected components and sets
+// Vertex[]->type_ext equal to component number >=1.
+int nComponents,i;
+
+  nComponents = 0;
+
+  Build ( False );
+
+  for (i=0;i<nVertices;i++)
+    Vertex[i]->type_ext = 0;
+
+  i = 0;
+  while (i<nVertices)  {
+    while (i<nVertices)
+      if (Vertex[i]->type_ext)  i++;
+                          else  break;
+    if (i<nVertices)  {
+      nComponents++;
+      markConnected ( i,nComponents );
+    }
+  }
+
+  return nComponents;
+
+}
+
+
+
 void  CGraph::Print()  {
 int i;
 
@@ -1277,8 +1326,10 @@ Boolean bondOrder;
 
 void  CGraph::mem_write ( pstr S, int & l )  {
 int     i,k;
+byte    Version=2;
 Boolean bondOrder=False;
 
+  ::mem_write_byte ( Version,S,l );
   ::mem_write ( bondOrder   ,S,l );
   ::mem_write ( name        ,S,l );
   ::mem_write ( nVertices   ,S,l );
@@ -1303,14 +1354,17 @@ Boolean bondOrder=False;
       k = 0;
       ::mem_write ( k,S,l );
     }
+
 }
 
 void  CGraph::mem_read ( cpstr S, int & l )  {
 int     i,k;
+byte    Version;
 Boolean bondOrder;
 
   FreeMemory();
 
+  ::mem_read_byte ( Version,S,l );
   ::mem_read ( bondOrder   ,S,l );
   ::mem_read ( name        ,S,l );
   ::mem_read ( nVertices   ,S,l );
@@ -1347,7 +1401,7 @@ Boolean bondOrder;
 
 }
 
-MakeStreamFunctions(CGraph)
+MakeStreamFunctions(CGraph);
 
 
 //  ==========================  CMatch  ============================
@@ -1527,7 +1581,7 @@ int i;
 }
 
 
-MakeStreamFunctions(CMatch)
+MakeStreamFunctions(CMatch);
 
 
 
@@ -2287,7 +2341,7 @@ int i;
   }
 }
 
-MakeStreamFunctions(CGraphMatch)
+MakeStreamFunctions(CGraphMatch);
 
 
 
