@@ -3,7 +3,7 @@
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2000-2008.
+//   Copyright (C) Eugene Krissinel 2000-2013.
 //
 //    This library is free software: you can redistribute it and/or
 //    modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    07.02.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -35,7 +35,7 @@
 //                  CMMCIFData     ( mmCIF data block  )
 //                  CMMCIFFile     ( mmCIF file        )
 //
-//  (C) E. Krissinel 2000-2010
+//  (C) E. Krissinel 2000-2013
 //
 //  =================================================================
 //
@@ -92,143 +92,6 @@ int  i,k,l,l1,l2;
 }
 
 
-/*  -----------   commented SortTags to be deleted after
-                  no bugs are discovered
-
-void  SortTags ( psvector tag, int len, ivector index )  {
-//   The idea is, the tags are created already sorted and
-// SortTags(..) should serve only in special cases, occasionally.
-// The bubble sorting should perform best in this situation
-// because of its N-complexity for sorted arrays.
-//   Note: index must contain valid indices on input.
-int     i,i1,i2;
-Boolean Done;
-//  for (i=0;i<len;i++)
-//    index[i] = i;
-  do  {
-    Done = True;
-    i1 = index[0];
-    for (i=1;i<len;i++)  {
-      i2 = index[i];
-      if (strcmp(tag[i1],tag[i2])>0)  {
-        index[i-1] = i2;
-        index[i]   = i1;
-        Done       = False;
-      } else
-        i1 = i2;
-    }
-  } while (!Done);
-}
-*/
-
-/*  ----------------------------------------------------------------
-    ANACHRONISM
-void  SortTags ( psvector tag, int len, ivector index )  {
-int i,j,k;
-  for (i=0;i<len;i++)
-    index[i] = i;
-  for (i=0;i<len-1;i++)
-    for (j=i+1;j<len;j++)
-      if (strcmp(tag[index[i]],tag[index[j]])>0)  {
-        k = index[i];
-        index[i] = index[j];
-        index[j] = k;
-      }
-}
-------------------------------------------------------------------- */
-
-
-/*  -----------------------------------------------------------------
-    THESE VERSIONS OF HEAPSORT FROM NUMERICAL RECIPIES DO NOT WORK!
-
-void  SortTags ( psvector tag, int len, ivector index )  {
-//   Sorts tags by alphabetical order. index[] will return
-// indices of ordered tags; it must come allocated. tag[]
-// does not change; ordered tags are then obtained as
-// tag[index[i]].
-int i,j,k,l,N;
-  for (i=0;i<len;i++)
-    index[i] = i;
-  if (len>1)  {
-    l = (len >> 1)+1;
-    k = len;
-    for (;;)  {
-      if (l>1) {
-        l--;
-        N = index[l-1];
-      } else  {
-        k--;
-        N = index[k];
-        index[k] = index[0];
-        if (k==1)  {
-          index[0] = N;
-          return;
-        }
-      }
-      i = l;
-      j = l+1;
-      while (j<=k)  {
-        if (j<k)  {
-          if (strcmp(tag[index[j-1]],tag[index[j]])<0)
-            j++;
-        }
-        if (strcmp(tag[N],tag[index[j-1]])<0)  {
-          index[i-1] = index[j-1];
-          i   = j;
-          j <<= 1;
-        } else
-          j = k+1;
-      }
-      index[i-1] = N;
-    }
-  }
-}
-
-void  SortTags ( psvector tag, int len, ivector index )  {
-//   Sorts tags by alphabetical order. index[] will return
-// indices of ordered tags; it must come allocated. tag[]
-// does not change; ordered tags are then obtained as
-// tag[index[i]].
-int i,j,k,l,N;
-  for (i=0;i<len;i++)
-    index[i] = i;
-  if (len>1)  {
-    l = len >> 1;
-    k = len-1;
-    for (;;)  {
-      if (l>0)
-        N = index[--l];
-      else  {
-        N = index[k];
-        index[k--] = index[0];
-        if (!k)  {
-          index[0] = N;
-          break;
-        }
-      }
-      i = l;
-      j = l+1;
-      while (j<=k)  {
-        if (j<k)  {
-          if (strcmp(tag[index[j]],tag[index[j+1]])<0)
-            j++;
-        }
-        if (strcmp(tag[N],tag[index[j]])<0)  {
-          index[i] = index[j];
-          i   = j;
-          j <<= 1;
-          j++;
-        } else
-          break;
-      }
-      index[i] = N;
-    }
-  }
-}
-
-------------------------------------------------------------------- */
-
-
 
 //  ======================  CMMCIFCategory  ==========================
 
@@ -239,11 +102,7 @@ CMMCIFCategory::CMMCIFCategory() : CStream()  {
 
 CMMCIFCategory::CMMCIFCategory ( cpstr N ) : CStream()  {
   InitMMCIFCategory();
-  if (N[0])  CreateCopy ( name,N );
-  else  {
-    CreateCopy ( name,pstr(" ") );
-    name[0] = char(1);  // no category name
-  }
+  SetCategoryName ( N );
 }
 
 CMMCIFCategory::CMMCIFCategory ( RPCStream Object )
@@ -275,6 +134,14 @@ int i;
   nAllocTags = 0;
 }
 
+void CMMCIFCategory::SetCategoryName ( cpstr N )  {
+  if (N[0])  CreateCopy ( name,N );
+  else  {
+    CreateCopy ( name,pstr(" ") );
+    name[0] = char(1);  // no category name
+  }
+}
+
 void CMMCIFCategory::ExpandTags ( int nTagsNew )  {
 int      i,nAT;
 psvector tag1;
@@ -353,8 +220,12 @@ int  CMMCIFCategory::GetTagNo ( cpstr ttag )  {
 //     <0 : the tag was not found, it could be inserted before
 //          (-RC-1)th element, where RC is the return value
 int l1,l2,l,k;
+
   if (!tag)    return -1;
+
   if (!index)  Sort();
+
+  l = 0;
   l1 = 0;
   l2 = nTags-1;
   k  = 1;
@@ -368,6 +239,7 @@ int l1,l2,l,k;
       break;
     }
   }
+
   if (k==0)  return index[l];    // is at RCth position
   k = strcasecmp ( ttag,tag[index[l1]] );
   if (k==0)  return index[l1];   // is at RCth position
@@ -377,7 +249,9 @@ int l1,l2,l,k;
     if (k==0)  return index[l2]; // is at RCth position
     if (k>0)   return -2-l2;     // would be at l2+1=(-RC-1)th position
   }
+
   return -2-l1;                  // would be at l1+1=(-RC-1)th position
+
 }
 
 int  CMMCIFCategory::AddTag ( cpstr ttag )  {
@@ -421,7 +295,7 @@ int i;
   }
 }
 
-Boolean CMMCIFCategory::CheckTags ( pstr * tagList )  {
+Boolean CMMCIFCategory::CheckTags ( cpstr * tagList )  {
 int i;
   i = 0;
   while (tagList[i][0])  {
@@ -481,7 +355,7 @@ int i;
   f.CreateReadVector ( index,0 );
 }
 
-MakeStreamFunctions(CMMCIFCategory)
+MakeStreamFunctions(CMMCIFCategory);
 
 
 
@@ -542,8 +416,7 @@ psvector f1;
   }
 }
 
-void CMMCIFStruct::AddField ( cpstr F, cpstr T,
-                              Boolean Concatenate )  {
+void CMMCIFStruct::AddField ( cpstr F, cpstr T, Boolean Concatenate )  {
 psvector field1;
 int      i,nAT;
 pstr     nf;
@@ -648,7 +521,7 @@ int  k = GetTagNo ( TName );
   if (!field)                return CIFRC_NoField;
   if (k<0)                   return CIFRC_NoTag;
   if (!field[k])             return CIFRC_NoField;
-  if (field[k][0]==char(2))  return CIFRC_NoField;
+  if (field[k][0]==char(2))  return CIFRC_NoData;
   R = strtod ( field[k],&endptr );
   if (endptr==field[k])  RC = CIFRC_WrongFormat;
   else  {
@@ -672,7 +545,7 @@ int  k = GetTagNo ( TName );
   if (!field[k])             return CIFRC_NoField;
   if (field[k][0]==char(2))  {
     if (field[k][1]=='.')  I = MinInt4;
-    return CIFRC_NoField;
+    return CIFRC_NoData;
   }
   I = mround ( strtod(field[k],&endptr) );
   if (endptr==field[k])  RC = CIFRC_WrongFormat;
@@ -730,6 +603,12 @@ char rS[100];
   AddField ( rS,T,False );
 }
 
+void CMMCIFStruct::PutReal ( realtype R, cpstr T, cpstr format )  {
+char rS[100];
+  sprintf  ( rS,format,R );
+  AddField ( DelSpaces(rS,' '),T,False );
+}
+
 void CMMCIFStruct::PutInteger ( int I, cpstr T )  {
 char iS[100];
   if (I>MinInt4)  {
@@ -756,21 +635,34 @@ CFile f;
     return False;
 }
 
+#define _max_output_line_width 256
+
 void CMMCIFStruct::WriteMMCIF ( RCFile f )  {
 int   i,j,k,l,m,n;
 pstr  F;
+
+  // calculate maximal length of tags
   l = 0;
   for (i=0;i<nTags;i++)
     l = IMax(l,strlen(tag[i]));
-  l += 1;
-  m  = 76-l;
+  l += 1;  // add one space separator
+
+  // calculate maximal space left for data
+  m  = _max_output_line_width - l;
+  // subtract category name width
   if (name[0]!=char(1))  m -= strlen(name);
+
+  // start outout
   f.LF();
-  for (i=0;i<nTags;i++)  {
+  for (i=0;i<nTags;i++)  {  // for each tag
+
+    // print category name, if not hidden, and dot
     if (name[0]!=char(1))  {
       f.Write ( name      );
       f.Write ( pstr(".") );
     }
+
+    // print tag, checking for duplicate tag flag
     F = strchr ( tag[i],'\1' );
     if (F)  {
       *F = char(0);
@@ -778,7 +670,9 @@ pstr  F;
       *F = '\1';
     } else
       f.Write ( tag[i] );
-    if (field[i])  {
+
+    // print field
+    if (field[i])  {  // field is defined
       F = field[i];
       if (strchr(F,'\n') || strstr(F,"\" "))  {
         f.Write ( pstr("\n;")   );
@@ -786,7 +680,7 @@ pstr  F;
         f.Write ( pstr("\n;\n") );
       } else {
         n = strlen(F);
-        if (n>m)
+        if (n>m)  // wrap around if field is too long
           f.Write ( pstr("\n ") );
         else {
           k = l-strlen(tag[i]);
@@ -805,13 +699,18 @@ pstr  F;
         } else
           f.WriteLine ( field[i] );
       }
-    } else  {
+
+    } else  {  // field if not defined, put question mark
+
       k = l-strlen(tag[i]);
       for (j=0;j<k;j++)
         f.Write ( pstr(" ") );
       f.WriteLine ( NODATA_Q );
+
     }
+
   }
+
 }
 
 void CMMCIFStruct::Copy ( PCMMCIFCategory Struct )  {
@@ -846,7 +745,7 @@ int i;
 }
 
 
-MakeStreamFunctions(CMMCIFStruct)
+MakeStreamFunctions(CMMCIFStruct);
 
 
 
@@ -1055,6 +954,12 @@ char rS[100];
   AddString ( rS );
 }
 
+void CMMCIFLoop::AddReal ( realtype R, cpstr format )  {
+char rS[100];
+  sprintf ( rS,format,R );
+  AddString ( DelSpaces(rS,' ') );
+}
+
 void CMMCIFLoop::AddInteger ( int I )  {
 char iS[100];
   if (I>MinInt4)  {
@@ -1119,6 +1024,48 @@ int k = GetTagNo ( TName );
   return field[nrow][k];
 }
 
+//  CopyString() does nothing if RC is not 0
+void CMMCIFLoop::CopyString   ( pstr  buf,   int maxlength,
+                                cpstr TName, int nrow, int & RC )  {
+pstr p;
+int  k;
+
+  if (RC)  return;
+
+  k = GetTagNo ( TName );
+  if (k<0)  {
+    RC = CIFRC_NoTag;
+    buf[0] = char(0);
+    return;
+  }
+  if ((nrow<0) || (nrow>=nRows))  {
+    RC = CIFRC_WrongIndex;
+    buf[0] = char(0);
+    return;
+  }
+  if (!field[nrow])  {
+    RC = CIFRC_NoField;
+    buf[0] = char(0);
+    return;
+  }
+  p = field[nrow][k];
+  if (!p)  {
+    RC = CIFRC_NoField;
+    buf[0] = char(0);
+    return;
+  }
+
+  // char(2) means the field was either '.' or '?'
+  if (p[0]==char(2))  {
+    buf[0] = p[0];
+    buf[1] = char(0);
+  } else
+    strncpy ( buf,p,IMin(maxlength,strlen(p)+1) );
+
+}
+
+
+
 int CMMCIFLoop::DeleteField ( cpstr TName, int nrow )  {
 int k = GetTagNo ( TName );
   if (k<0)  return CIFRC_NoTag;
@@ -1167,6 +1114,50 @@ int  k = GetTagNo ( TName );
   return 0;
 }
 
+void CMMCIFLoop::CopyReal ( realtype & R, cpstr TName, int nrow,
+                            int & RC )  {
+pstr endptr;
+int  k;
+
+  if (RC)  return;
+
+//  R = 0.0;
+  k = GetTagNo ( TName );
+
+  if (k<0)                              RC = CIFRC_NoTag;
+  else if ((nrow<0) || (nrow>=nRows))   RC = CIFRC_WrongIndex;
+  else if (!field[nrow])                RC = CIFRC_NoField;
+  else if (!field[nrow][k])             RC = CIFRC_NoField;
+  else if (field[nrow][k][0]==char(2))  RC = CIFRC_NoField;
+  else  {
+    R = strtod ( field[nrow][k],&endptr );
+    if (endptr==field[nrow][k])  RC = CIFRC_WrongFormat;
+  }
+
+}
+
+void CMMCIFLoop::CopyInteger ( int & I, cpstr TName, int nrow,
+                               int & RC )  {
+pstr endptr;
+int  k;
+
+  if (RC)  return;
+
+  I = 0;
+  k = GetTagNo ( TName );
+
+  if (k<0)                              RC = CIFRC_NoTag;
+  else if ((nrow<0) || (nrow>=nRows))   RC = CIFRC_WrongIndex;
+  else if (!field[nrow])                RC = CIFRC_NoField;
+  else if (!field[nrow][k])             RC = CIFRC_NoField;
+  else if (field[nrow][k][0]==char(2))  RC = CIFRC_NoField;
+  else  {
+    I = mround ( strtod ( field[nrow][k],&endptr ) );
+    if (endptr==field[nrow][k])  RC = CIFRC_WrongFormat;
+  }
+
+}
+
 int  CMMCIFLoop::GetInteger ( int & I, cpstr TName, int nrow,
                               Boolean Remove )  {
 pstr endptr;
@@ -1337,13 +1328,19 @@ char S[10];
 }
 
 
-void CMMCIFLoop::PutReal ( realtype R, cpstr T, int nrow,
-                           int prec )  {
+void CMMCIFLoop::PutReal ( realtype R, cpstr T, int nrow, int prec )  {
 char rS[100];
   sprintf ( rS,"%.*g",prec,R );
   PutString ( rS,T,nrow );
 }
 
+void CMMCIFLoop::PutReal ( realtype R, cpstr T, int nrow,
+                           cpstr format )  {
+char rS[100];
+  sprintf ( rS,format,R );
+  PutString ( DelSpaces(rS,' '),T,nrow );
+}
+
 void CMMCIFLoop::PutInteger ( int I, cpstr T, int nrow )  {
 char iS[100];
   if (I>MinInt4)  {
@@ -1420,7 +1417,10 @@ void CMMCIFLoop::WriteMMCIF ( RCFile f )  {
 int     i,j,k,m,n;
 ivector l;
 pstr    F;
+
+  // write loop keyword
   f.Write ( pstr("\nloop_\n") );
+
   GetVectorMemory ( l,nTags,0 );
   k = 0;
   for (i=0;i<nTags;i++)  {
@@ -1454,7 +1454,7 @@ pstr    F;
       }
     l[i] = IMax(l[i],1);
     k += l[i]+1;
-    if (k>76)  {
+    if (k>_max_output_line_width)  {
       l[i] = -l[i];
       k = 0;
     }
@@ -1467,7 +1467,7 @@ pstr    F;
       k = l[j];   // length of the field
       if (k<0)  k = -k;
       m += k+1;
-      if (m>76)  {
+      if (m>_max_output_line_width)  {
         f.LF();
         m = k+1;
       } else
@@ -1554,8 +1554,8 @@ void CMMCIFLoop::write ( RCFile f )  {
 int i,j;
   CMMCIFCategory::write ( f );
   f.WriteInt ( &nRows );
-  if ((nTags>0) && (nRows>0))
-    for (i=0;i<nRows;i++)
+  if ((nTags>0) && (nRows>0)){
+    for (i=0;i<nRows;i++){
       if (field[i])  {
         j = 1;
         f.WriteInt ( &j );
@@ -1565,6 +1565,8 @@ int i,j;
         j = 0;
         f.WriteInt ( &j );
       }
+    }
+  }
   f.WriteInt ( &iColumn );
 }
 
@@ -1680,7 +1682,7 @@ void  CMMCIFData::RemoveFlag ( int F )  {
   flags &= ~F;
 }
 
-void  CMMCIFData::SetWrongFields ( pstr * cats, pstr * tags )  {
+void  CMMCIFData::SetWrongFields ( cpstr *cats, cpstr *tags )  {
 int i,lc,lt;
   FreeWrongFields();
   if ((!cats) || (!tags))  return;
@@ -1716,12 +1718,16 @@ int i;
   return False;
 }
 
+#define _max_buf_len   500
+
+static char  _err_string[_max_buf_len+1];
+static int   _err_line;
+
 
 int  CMMCIFData::ReadMMCIFData ( cpstr FName, byte gzipMode )  {
 CFile f;
-char  S[500];
-int   lcount;
-int   RC;
+char  S[_max_buf_len+1];
+int   RC,lcount;
   f.assign ( FName,True,False,gzipMode );
   if (f.reset(True))  {
     S[0]   = char(0);
@@ -1730,6 +1736,8 @@ int   RC;
     f.shut();
     return RC;
   } else  {
+    _err_string[0] = char(0);
+    _err_line      = 0;
     Warning = CIFRC_CantOpenFile;
     return CIFRC_CantOpenFile;
   }
@@ -1753,20 +1761,22 @@ pstr L;
     p = &(S[0]);
     while ((*p==' ') || (*p==char(9)))  p++;
     if (strncmp(p,"data_",5))  {
-      f.ReadLine ( S, 10000 );
+      f.ReadLine ( S,_max_buf_len );
       lcount++;
       p = NULL;
     }
   } while ((!p) && (!f.FileEnd()));
 
   if (!p)  {
+    strcpy ( _err_string,S );
+    _err_line = lcount;
     if (flags & CIFFL_PrintWarnings)
       printf ( "\n **** mmCIF READ ERROR "
                "<<line %i: no 'data_XXXX' tag found>>\n",lcount );
     return CIFRC_NoDataLine;
   }
 
-  llen = 11000;
+  llen = _max_buf_len;
   L    = new char[llen];
   i    = 0;
   p   += 5;
@@ -1795,10 +1805,13 @@ pstr L;
         break;
       } else  {
         // if got to here, the file is corrupted
+        strcpy ( _err_string,S );
+        _err_line = lcount;
         Warning |= CIFW_UnrecognizedItems;
         if (flags & CIFFL_PrintWarnings)
           printf ( "\n **** mmCIF READ WARNING "
-                   "<<line %i: unrecognized string>>\n%s\n",lcount,S );
+                   "<<line %i: unrecognized string>>\n%s\n",
+                   lcount,S );
         while ((*p) && (*p!=' ') && (*p!=char(9)))
           if (*p=='#')  *p = char(0);
                   else  p++;
@@ -1813,7 +1826,7 @@ pstr L;
 
     if (!(*p))  {
       if (!f.FileEnd())  {
-        f.ReadLine ( S, 10000 );
+        f.ReadLine ( S,_max_buf_len );
         lcount++;
         p = &(S[0]);
       } else
@@ -1823,7 +1836,6 @@ pstr L;
   }
 
   if (L)  delete[] L;
-  L = 0;
 
   Optimize();  // get rid of over-allocated fields.
 
@@ -1861,6 +1873,8 @@ int           RC,i;
   } else  {
     Struct = PCMMCIFStruct(Category[i]);
     if (Struct->GetCategoryID()!=MMCIF_Struct)  {
+      strcpy ( _err_string,S );
+      _err_line = lcount;
       Warning |= CIFW_NotAStructure;
       if (flags & CIFFL_PrintWarnings)
         printf ( "\n **** mmCIF READ WARNING "
@@ -1894,6 +1908,8 @@ int           RC,i;
   RC = GetField ( f,S,L,p,lcount,llen );
 
   if (RC)  {
+    strcpy ( _err_string,S );
+    _err_line = lcount;
     Warning |= CIFW_MissingField;
     if (flags & CIFFL_PrintWarnings)
       printf ( "\n **** mmCIF READ WARNING "
@@ -1910,7 +1926,9 @@ int           RC,i;
       tagNo++;
       ParamStr ( T,pstr("\1"),tagNo );
     } else  {
-      Warning |= CIFW_DuplicatedTag;
+      strcpy ( _err_string,S );
+      _err_line = lcount;
+      Warning |= CIFW_DuplicateTag;
       if (flags & CIFFL_PrintWarnings)
         printf ( "\n **** mmCIF READ WARNING "
                  "<<line %i: duplicated tag>>\n%s\n",lcount,S );
@@ -1928,6 +1946,8 @@ char        T[100];
 Boolean     Repeat,WrongField;
 int         RC,i,nC;
 
+  RC = 0;
+
   p += 5;  // skip 'loop_' tag
 
   loopNo++;
@@ -1968,6 +1988,8 @@ int         RC,i,nC;
         if ((i!=nC) && (nC>=0))  {  // empty loop; most probably
                                     // a corrupted file
           p = p1;   // play back to last category
+          strcpy ( _err_string,S );
+          _err_line = lcount;
           Warning |= CIFW_EmptyLoop;
           if (flags & CIFFL_PrintWarnings)
             printf ( "\n **** mmCIF READ WARNING "
@@ -2028,10 +2050,12 @@ int         RC,i,nC;
               ParamStr ( T,pstr("\1"),tagNo );
               Loop->AddTag(T);
             } else  {
-              Warning |= CIFW_DuplicatedTag;
+              strcpy ( _err_string,S );
+              _err_line = lcount;
+              Warning |= CIFW_DuplicateTag;
               if (flags & CIFFL_PrintWarnings)
                 printf ( "\n **** mmCIF READ WARNING "
-                         "<<line %i: duplicated tag>>\n%s\n",lcount,S );
+                         "<<line %i: duplicate tag>>\n%s\n",lcount,S );
             }
           }
         }
@@ -2047,10 +2071,12 @@ int         RC,i,nC;
     } else if (!(*p) || (*p=='#'))  {
       Repeat = !f.FileEnd();
       if (Repeat)  {
-        f.ReadLine ( S, 10000 );
+        f.ReadLine ( S,_max_buf_len );
         lcount++;
         p = &(S[0]);
       } else  {
+        strcpy ( _err_string,S );
+        _err_line = lcount;
         Warning |= CIFW_UnexpectedEOF;
         if (flags & CIFFL_PrintWarnings)
           printf ( "\n **** mmCIF READ WARNING "
@@ -2068,7 +2094,7 @@ int         RC,i,nC;
       if (!(*p) || (*p=='#'))  {
         Repeat = !f.FileEnd();
         if (Repeat)  {
-          f.ReadLine ( S, 10000 );
+          f.ReadLine ( S,_max_buf_len );
           lcount++;
           p = &(S[0]);
         }
@@ -2088,6 +2114,8 @@ int         RC,i,nC;
       }
     } while (Repeat);
     if ((Loop->iColumn!=0) || (RC))  {
+      strcpy ( _err_string,S );
+      _err_line = lcount;
       Warning |= CIFW_LoopFieldMissing;
       if (flags & CIFFL_PrintWarnings)
         printf ( "\n **** mmCIF READ WARNING "
@@ -2119,7 +2147,7 @@ char    c;
       Repeat = !f.FileEnd();
       if (Repeat)  {
         // take the next line
-        f.ReadLine ( S, 10000 );
+        f.ReadLine ( S,_max_buf_len );
         lcount++;
         p = &(S[0]);
         Repeat = (*p!=';');
@@ -2140,7 +2168,7 @@ char    c;
     strcpy ( L,p );    // take first line of the field
     flen = strlen(L);
     while (!f.FileEnd())  {
-      f.ReadLine ( S, 10000 );
+      f.ReadLine ( S,_max_buf_len );
       lcount++;
       p = &(S[0]);
       if (*p==';')  {  // multiline field terminated
@@ -2258,11 +2286,12 @@ int  CMMCIFData::GetCategoryNo ( cpstr cname )  {
 //          (-RC-1)th element, where RC is the return value
 int l1,l2,l,k;
 
-  l = 0; // may become undefined in not initialized
-
   if (!Category) return -1;
+
   if (!index)    Sort();
+
   if (cname[0])  {
+    l  = 0;
     l1 = 0;
     l2 = nCategories-1;
     k  = 1;
@@ -2436,7 +2465,7 @@ int i;
   else  return NULL;
 }
 
-PCMMCIFLoop CMMCIFData::FindLoop ( pstr * tagList )  {
+PCMMCIFLoop CMMCIFData::FindLoop ( cpstr * tagList )  {
 int i;
   for (i=0;i<nCategories;i++)
     if (Category[i])  {
@@ -3253,28 +3282,26 @@ int i;
 }
 
 
-static char  S[500];
-static int   lcount;
-
 pstr GetMMCIFInputBuffer ( int & LineNo )  {
-  LineNo         = lcount;
-  S[sizeof(S)-1] = char(0);
-  return S;
+  LineNo = _err_line;
+  _err_string[sizeof(_err_string)-1] = char(0);
+  return _err_string;
 }
 
 int  CMMCIFFile::ReadMMCIFFile ( cpstr FName, byte gzipMode )  {
 CFile       f;
 PCMMCIFData CIF;
-int         RC;
+char        S[500];
+int         RC,lcount;
 
   FreeMemory();
 
   CIF = NULL;
   f.assign ( FName,True,False,gzipMode );
   if (f.reset(True))  {
-    S[0]       = char(0);
-    lcount     = 0;
-    RC         = 0;
+    S[0]   = char(0);
+    lcount = 0;
+    RC     = 0;
     while ((!RC) && (!f.FileEnd()))  {
       if (!CIF)  CIF = new CMMCIFData();
       CIF->SetPrintWarnings ( PrintWarnings );
@@ -3346,8 +3373,11 @@ int  CMMCIFFile::GetCIFDataNo ( cpstr DName )  {
 //     <0 : the DName was not found, it could be inserted before
 //          (-RC-1)th element, where RC is the return value
 int l1,l2,l,k;
+
   if (!data)   return -1;
   if (!index)  Sort();
+
+  l  = 0;
   l1 = 0;
   l2 = nData-1;
   k  = 1;
@@ -3361,6 +3391,7 @@ int l1,l2,l,k;
       break;
     }
   }
+
   if (k==0)  return index[l];    // is at RCth position
   k = strcasecmp ( DName,data[index[l1]]->name );
   if (k==0)  return index[l1];   // is at RCth position
@@ -3370,7 +3401,9 @@ int l1,l2,l,k;
     if (k==0)  return index[l2]; // is at RCth position
     if (k>0)   return -2-l2;     // would be at l2+1=(-RC-1)th position
   }
+
   return -2-l1;                  // would be at l1+1=(-RC-1)th position
+
 }
 
 PCMMCIFData  CMMCIFFile::GetCIFData ( int dataNo )  {
@@ -3438,6 +3471,34 @@ int  i1,i;
 }
 
 
+int CMMCIFFile::DeleteMMCIFData ( cpstr DName )  {
+int dataNo = GetCIFDataNo ( DName );
+
+  if (dataNo>=0)  return DeleteMMCIFData ( dataNo );
+  return dataNo;
+
+}
+
+int CMMCIFFile::DeleteMMCIFData ( int dataNo )  {
+int i;
+
+  if ((0<=dataNo) && (dataNo<nData))  {
+
+    if (data[dataNo])  delete data[dataNo];
+    for (i=dataNo+1;i<nData;i++)
+      data[i-1] = data[i];
+    nData--;
+
+    Sort();
+
+    return 0;
+
+  }
+
+  return -nData;
+
+}
+
 void CMMCIFFile::Copy  ( PCMMCIFFile File )  {
 int i;
   FreeMemory();
@@ -3493,38 +3554,46 @@ MakeStreamFunctions(CMMCIFFile)
 
 
 int  isCIF ( cpstr FName, byte gzipMode )  {
-CFile   f;
-char    S[256];
-Boolean Done;
-pstr    p;
-int     m;
+CFile f;
+int   rc;
 
   f.assign ( FName,True,False,gzipMode );
-  m = sizeof(S)-1;
   if (f.reset(True))  {
-    f.ReadLine ( S,m );
-    S[m] = char(0);
-    Done = False;
-    while (!Done)  {
-      p = &(S[0]);
-      while ((*p==' ') || (*p==char(9)))  p++;
-      Done = !strncmp(p,"data_",5);
-      if (!Done)  {
-        if (f.FileEnd())  {
-          Done = True;
-          p    = NULL;
-        } else  {
-          f.ReadLine ( S,sizeof(S)-1 );
-          S[m] = char(0);
-        }
+    rc = isCIF ( f );
+    f.shut();
+  } else
+    rc = -1;
+
+  return rc;
+
+}
+
+int  isCIF ( RCFile f )  {
+char    S[_max_buf_len+1];
+Boolean Done;
+pstr    p;
+
+  f.ReadLine ( S,_max_buf_len );
+  S[_max_buf_len] = char(0);
+  Done = False;
+  while (!Done)  {
+    p = &(S[0]);
+    while ((*p==' ') || (*p==char(9)))  p++;
+    Done = !strncmp(p,"data_",5);
+    if (!Done)  {
+      if (f.FileEnd())  {
+        Done = True;
+        p    = NULL;
+      } else  {
+        f.ReadLine ( S,_max_buf_len );
+        S[_max_buf_len] = char(0);
       }
     }
-    f.shut();
-    if (!p)  return 1;
-    if (!strncmp(p,"data_",5))  return 0;
-                          else  return 1;
-  } else
-    return -1;
+  }
+
+  if (!p)  return 1;
+  if (!strncmp(p,"data_",5))  return 0;
+                        else  return 1;
 
 }
 
@@ -3560,8 +3629,8 @@ pstr InputLine;
     else if (RC & CIFW_NotALoop)
       sprintf ( M,"a structure is used as a loop on line %i\n%s",
                 LineNo,InputLine );
-    else if (RC & CIFW_DuplicatedTag)
-      sprintf ( M,"duplicated tag was found on line %i\n%s",
+    else if (RC & CIFW_DuplicateTag)
+      sprintf ( M,"duplicate tag was found on line %i\n%s",
                 LineNo,InputLine );
     else
       sprintf ( M,"undocumented warning issued for line %i\n%s",
