@@ -5,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or
-//    modify it under the terms of the GNU Lesser General Public
-//    License version 3, modified in accordance with the provisions
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser
-//    General Public License along with this library. If not, copies
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -618,6 +618,24 @@ int n,i;
   return n;
 }
 
+int CSBStructure::GetAtomNo_nss ( cpstr sca_name )  {
+// disregards leading and trailing spaces
+AtomName name_0;
+AtomName name_i;
+int n,i;
+  strcpy_css ( name_0,sca_name );
+//  printf ( " name_0 = '%s'\n",name_0 );
+  n = 0;
+  for (i=0;(i<nAtoms) && (!n);i++)
+    if (Atom[i])  {
+      strcpy_css ( name_i,Atom[i]->sca_name );
+//      printf ( " name_i = '%s'\n",name_i );
+      if (!strcmp(name_0,name_i))
+        n = i+1;
+    }
+  return n;
+}
+
 PCSBAtom CSBStructure::GetAtom ( cpstr sca_name )  {
 int n;
   n = GetAtomNo ( sca_name );
@@ -682,7 +700,7 @@ Boolean done;
                     k    = j;
                     done = True;
                   }
-                }
+                } 
               }
             }
             j++;
@@ -760,7 +778,7 @@ int      i,j,k;
 
   //  2. Calculate transformation matrix (to be applied to A1)
 
-  det = A[1][1]*A[2][2]*A[3][3] +
+  det = A[1][1]*A[2][2]*A[3][3] + 
         A[1][2]*A[2][3]*A[3][1] +
         A[2][1]*A[3][2]*A[1][3] -
         A[1][3]*A[2][2]*A[3][1] -
@@ -1397,7 +1415,7 @@ int            structNo,i,k,RC;
       } else
         RC = SBASE_FileNotFound;
     }
-  } else
+  } else 
     RC = SBASE_StructNotFound;
 
   if (RC==SBASE_Ok)  {
@@ -1485,12 +1503,11 @@ int CSBase0::GetStructNo ( cpstr compoundID )  {
 int  l1,l2,l,k;
 char id[20];
 
-  l = k = 0; // initialize, else may become undefined
-
   strcpy_css ( id,compoundID );
 
+  k = -1;
+
   if (nStructures<=3)  {
-    k = -1;
     for (l=0;(l<nStructures) && (k<0);l++)
       if (!strcasecmp(Index[l]->compoundID,id))
         k = l;
@@ -1498,6 +1515,7 @@ char id[20];
     return SBASE_StructNotFound;
   }
 
+  l  = 0;
   l1 = 0;
   l2 = nStructures-1;
   while (l1<l2-1)  {
@@ -1633,7 +1651,7 @@ int           structNo;
 }
 
 
-PCResidue CSBase0::makeCResidue ( cpstr compoundID,
+PCResidue CSBase0::makeCResidue ( cpstr compoundID, 
                                   PCFile     structFile,
                                   Boolean    includeHydrogens,
                                   Boolean    makeTer )  {
@@ -1648,7 +1666,7 @@ PCResidue     Res;
     return NULL;
 }
 
-PCResidue CSBase0::makeCResidue ( int     structNo,
+PCResidue CSBase0::makeCResidue ( int     structNo, 
                                   PCFile  structFile,
                                   Boolean includeHydrogens,
                                   Boolean makeTer )  {
@@ -2166,7 +2184,7 @@ int           i,j,rc,natoms;
     } else
       rc =  SBASE_NoAtomsFound;
     delete SBS;
-  } else
+  } else  
     rc = SBASE_StructNotFound;
 
   return rc;
@@ -2218,7 +2236,7 @@ int           i,j,k,n,natoms;
                     }
                 }
             }
-          }
+          } 
         }
         j++;
         while (j<nRes)
@@ -2258,7 +2276,7 @@ PPCResidue    Res;
 PPCChain      chain;
 PCMMDBManager MMDB;
 int           rc,selHnd,i,nRes,nChains;
-
+ 
   rc   = SBASE_Ok;
   MMDB = PCMMDBManager(model->GetCoordHierarchy());
 
@@ -2581,7 +2599,7 @@ Boolean       complH,complNH,complXT;
           aname = A1[i]->GetAtomName();
           if (strcmp(A1[i]->GetElementName(),Hydrogen))  {
             //  a non-hydrogen atom, check if it should be added
-            if (complNH &&
+            if (complNH &&  
                      (complXT || (strcmp(aname," OXT"))))
               A3[k] = A1[i];
           } else if (complH)  {
@@ -2894,7 +2912,7 @@ PCFile        structFile;
     SBS = ldStructure[Index[structNo]->loadPos];
 
   } else  {
-
+  
     structFile = GetStructFile();
     if (!structFile)  return SBASE_FileNotFound;
 
@@ -2954,7 +2972,7 @@ PCFile        structFile;
     }
     j = 0;
     while ((j<nHAtoms) && (Hconnect[j]>=0)) j++;
-    if (j<nHAtoms)
+    if (j<nHAtoms) 
       rc = SBASE_ConnectivityError;
   }
 
@@ -3023,7 +3041,7 @@ int     structNo,i, a1,a2;
       }
     }
   }
-
+  
   if (Index[structNo]->loadPos<0)  delete G;
 
   return SBASE_Ok;
