@@ -450,7 +450,7 @@ byte Version;
       f.ReadReal ( &(T[i][j]) );
 }
 
-MakeStreamFunctions(CSymOp)
+MakeStreamFunctions(CSymOp);
 
 
 
@@ -558,17 +558,172 @@ int   i,RC;
 
 int  CSymOps::SetGroup ( cpstr SpaceGroup,
                          cpstr syminfo_lib )  {
-CFile f;
-pstr  p;
-char  S[500];
-char  G[100];
-char  O[100];
-mat44 T1,T2,T3;
-int   i,j,k,l,m,RC;
-int   npops,ncops;
-long  symop_start;
+CFile    f;
+pstr     p;
+psvector lines,lines1;
+char     S[500];
+char     G[100];
+char     O[100];
+mat44    T1,T2,T3;
+int      i,j,k,l,m,n,RC;
+int      nlines,npops,ncops;
 
-  symop_start = 0;  // else may be used uninitialized 
+  FreeMemory();
+
+  npops = 0;
+  ncops = 0;
+
+  CreateCopy ( SpGroup,SpaceGroup );
+
+  if (!syminfo_lib)          p = pstr(syminfo_file);
+  else if (!syminfo_lib[0])  p = pstr(syminfo_file);
+                       else  p = pstr(syminfo_lib);
+  f.assign ( p,True );
+  if (!f.reset(True))  {
+    p = getenv ( "SYMINFO" );
+    if (p)
+      strcpy ( S,p );
+    else  {
+      p = getenv ( "CLIBD" );
+      if (p)  {
+        strcpy ( S,p );
+        if (S[strlen(S)-1]!='/')  strcat ( S,"/" );
+        strcat ( S,"syminfo.lib" );
+      } else
+        strcpy ( S,"syminfo.lib" );
+    }
+    f.assign ( S,True );
+    if (!f.reset(True))  return SYMOP_NoLibFile;
+  }
+
+
+  if (strncasecmp(SpGroup,"Hall:",5))  {
+    // normal space group symbol on input
+    strcpy ( G," '"    );
+    strcat ( G,SpGroup );
+    strcat ( G,"'"     );
+    S[0] = char(0);
+    while (!f.FileEnd() && !(strstr(S,G) && (strstr(S,"symbol xHM") ||
+            strstr(S,"symbol old"))))
+      f.ReadLine ( S,sizeof(S) );
+  } else  {
+    // hall descriptor on input
+    strcpy ( G," ' " );
+    p = &(SpGroup[5]);
+    while (*p==' ')  p++;
+    strcat ( G,p     );
+    strcat ( G,"'"   );
+    S[0] = char(0);
+    while (!f.FileEnd() && !(strstr(S,G) && strstr(S,"symbol Hall")))
+      f.ReadLine ( S,sizeof(S) );
+  }
+  if (f.FileEnd())  {
+    f.shut();
+    return SYMOP_UnknownSpaceGroup;
+  }
+
+  // found spacegroup, move to symop lines
+  while (!f.FileEnd() && (!strstr(S,"symop")))
+    f.ReadLine ( S,sizeof(S) );
+
+  nlines = 256;
+  GetVectorMemory ( lines,nlines,0 );
+  for (i=0;i<nlines;i++)
+    lines[i] = NULL;
+  n = 0;
+  CreateCopy ( lines[n++],S );
+
+  // count primitive operators
+  while (!f.FileEnd() && (strstr(S,"symop"))) {
+    npops++;
+    f.ReadLine ( S,sizeof(S) );
+    if (n>=nlines)  {
+      nlines += + 256;
+      GetVectorMemory ( lines1,nlines,0 );
+      for (i=0;i<n;i++)
+        lines1[i] = lines[i];
+      for (i=n;i<nlines;i++)
+        lines1[i] = NULL;
+      FreeVectorMemory ( lines,0 );
+      lines = lines1;
+    }
+    CreateCopy ( lines[n++],S );
+  }
+
+  // count centering operators
+  while (!f.FileEnd() && (strstr(S,"cenop"))) {
+    ncops++;
+    f.ReadLine ( S,sizeof(S) );
+    if (n>=nlines)  {
+      nlines += + 256;
+      GetVectorMemory ( lines1,nlines,0 );
+      for (i=0;i<n;i++)
+        lines1[i] = lines[i];
+      for (i=n;i<nlines;i++)
+        lines1[i] = NULL;
+      FreeVectorMemory ( lines,0 );
+      lines = lines1;
+    }
+    CreateCopy ( lines[n++],S );
+  }
+
+  Nops = npops*ncops;
+  SymOp = new PCSymOp[Nops];
+  RC    = SYMOP_Ok;
+
+  n = 0;  // start second pass here
+
+  // read primitive operators
+  for (i=0;(i<npops) && (!RC);i++)  {
+    SymOp[i] = new CSymOp();
+    RC = SymOp[i]->SetSymOp ( lines[n++]+6 );
+  }
+
+  // loop over non-trivial centering operators, and for each loop
+  // over primtive operators
+  for (i=1;(i<ncops) && (!RC);i++)  {
+    n++;  // this also skips the identity operator
+    for (j=0;(j<npops) && (!RC);j++)  {
+      SymOp[i*npops+j] = new CSymOp();
+      RC = SymOp[i*npops+j]->SetSymOp ( lines[n]+6 );
+      SymOp[i*npops+j]->GetTMatrix(T1);
+      SymOp[j]->GetTMatrix(T2);
+      for (k=0;k<4;k++)
+        for (l=0;l<4;l++) {
+          T3[k][l] = 0.0;
+          for (m=0;m<4;m++)
+            T3[k][l] += T1[k][m]*T2[m][l];
+        }
+      for (k=0;k<3;k++)                  // kdc fix
+        T3[k][3] -= floor ( T3[k][3] );  // kdc fix
+      SymOp[i*npops+j]->CompileOpTitle ( O,T3,False );
+      SymOp[i*npops+j]->SetSymOp ( O );
+    }
+  }
+
+  f.shut();
+
+  for (i=0;i<nlines;i++)
+    if (lines[i])  delete[] lines[i];
+  FreeVectorMemory ( lines,0 );
+
+  return RC;
+
+}
+
+/*
+int  CSymOps::SetGroup ( cpstr SpaceGroup,
+                         cpstr syminfo_lib )  {
+CFile  f;
+pstr   p;
+char   S[500];
+char   G[100];
+char   O[100];
+mat44  T1,T2,T3;
+int    i,j,k,l,m,RC;
+int    npops,ncops;
+long   symop_start;
+//fpos_t symop_start;
 
   FreeMemory();
 
@@ -626,6 +781,7 @@ long  symop_start;
   // found spacegroup, move to symop lines
   while (!f.FileEnd() && (!strstr(S,"symop"))) {
     symop_start = f.Position();
+//#    fgetpos ( f.GetHandle(),&symop_start );
     f.ReadLine ( S,sizeof(S) );
   }
   // count primitive operators
@@ -640,6 +796,7 @@ long  symop_start;
   }
   Nops = npops*ncops;
   f.seek(symop_start);
+//#  fsetpos ( f.GetHandle(),&symop_start );
   SymOp = new PCSymOp[Nops];
   RC    = SYMOP_Ok;
 
@@ -663,11 +820,11 @@ long  symop_start;
       SymOp[i*npops+j]->GetTMatrix(T1);
       SymOp[j]->GetTMatrix(T2);
       for (k=0;k<4;k++)
-	for (l=0;l<4;l++) {
+        for (l=0;l<4;l++) {
           T3[k][l] = 0.0;
-  	  for (m=0;m<4;m++)
+          for (m=0;m<4;m++)
             T3[k][l] += T1[k][m]*T2[m][l];
-	}
+        }
       for (k=0;k<3;k++)                  // kdc fix
         T3[k][3] -= floor ( T3[k][3] );  // kdc fix
       SymOp[i*npops+j]->CompileOpTitle(O,T3,False);
@@ -680,6 +837,7 @@ long  symop_start;
   return RC;
 
 }
+*/
 
 void CSymOps::Reset()  {
 // removes all symmetry operations
