@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    23.06.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -37,6 +37,7 @@
 //                  CSource
 //                  CKeyWords
 //                  CExpData
+//                  CMdlType
 //                  CAuthor
 //                  CRevData
 //                  CSupersede
@@ -45,7 +46,7 @@
 //                  CBiomolecule
 //                  CMMDBTitle  ( MMDB title section )
 //
-//  (C) E. Krissinel 2000-2010
+//  (C) E. Krissinel 2000-2013
 //
 //  =================================================================
 //
@@ -801,6 +802,57 @@ MakeStreamFunctions(CExpData)
 
 
 
+
+//  ===================  CMdlType  ======================
+
+CMdlType::CMdlType() : CContString()  {
+  InitMdlType();
+}
+
+CMdlType::CMdlType ( cpstr S ) : CContString()  {
+  InitMdlType();
+  ConvertPDBASCII ( S );
+}
+
+CMdlType::CMdlType ( RPCStream Object ) : CContString(Object)  {
+  InitMdlType();
+}
+
+CMdlType::~CMdlType() {}
+
+void  CMdlType::InitMdlType()  {
+  CreateCopy ( CIFCategory,CIFCAT_EXPTL  );
+  CreateCopy ( CIFTag     ,CIFTAG_METHOD );
+}
+
+int  CMdlType::ConvertPDBASCII ( cpstr S )  {
+  if (strlen(S)>10)
+       CreateCopy ( Line,&(S[10])  );
+  else CreateCopy ( Line,pstr(" ") );
+  return 0;
+}
+
+void  CMdlType::PDBASCIIDump ( pstr S, int N )  {
+  if (N==0)  strcpy  ( S,"MDLTYP    " );
+       else  sprintf ( S,"MDLTYP  %2i",N+1 );
+  strcat ( S,Line );
+}
+
+void  CMdlType::write ( RCFile f )  {
+byte Version=1;
+  f.WriteByte ( &Version );
+  CContString::write ( f );
+}
+
+void  CMdlType::read ( RCFile f )  {
+byte Version;
+  f.ReadByte ( &Version );
+  CContString::read ( f );
+}
+
+MakeStreamFunctions(CMdlType)
+
+
 //  ===================  CAuthor  ======================
 
 CAuthor::CAuthor() : CContString()  {
@@ -1821,6 +1873,7 @@ void  CMMDBTitle::FreeMemory ( Boolean keepBiomolecules )  {
   Source   .FreeContainer();
   KeyWords .Delete       ();
   ExpData  .FreeContainer();
+  MdlType  .FreeContainer();
   Author   .FreeContainer();
   RevData  .FreeContainer();
   Supersede.FreeContainer();
@@ -1919,6 +1972,11 @@ PCContainerClass ContainerClass;
     ContainerClass = new CExpData(PDBString);
     ExpData.AddData ( ContainerClass );
 
+  } else if (!strncmp(PDBString,"MDLTYPE",6))  {
+
+    ContainerClass = new CMdlType(PDBString);
+    MdlType.AddData ( ContainerClass );
+
   } else if (!strncmp(PDBString,"AUTHOR",6))  {
 
     ContainerClass = new CAuthor(PDBString);
@@ -1948,6 +2006,7 @@ PCContainerClass ContainerClass;
     return Error_WrongSection;
 
   //  check for ID code in columns 73-80
+
   if (!col73)  {
     if (('0'<=idCode[0]) && (idCode[0]<='9'))  {
       if (!strncasecmp(idCode,&(PDBString[72]),4))
@@ -1963,6 +2022,10 @@ PCTitleContainer CMMDBTitle::GetRemarks()  {
   return &Remark;
 }
 
+PCTitleContainer CMMDBTitle::GetJournal()  {
+  return &Journal;
+}
+
 realtype CMMDBTitle::GetResolution()  {
 //  returns -1.0 if there is no resolution record in the file
 PCRemark rem;
@@ -2115,6 +2178,7 @@ int  RC;
   Source  .GetCIF ( CIF,ClassID_Source    );
   KeyWords.GetCIF ( CIF );
   ExpData .GetCIF ( CIF,ClassID_ExpData   );
+  MdlType .GetCIF ( CIF,ClassID_MdlType   );
   Author  .GetCIF ( CIF,ClassID_Author    );
   RC = RevData.GetCIF ( CIF,ClassID_RevData );
   if (!RC)  {
@@ -2276,6 +2340,7 @@ char  PDBString[100];
   Source   .PDBASCIIDump ( f );
   KeyWords .PDBASCIIDump ( f );
   ExpData  .PDBASCIIDump ( f );
+  MdlType  .PDBASCIIDump ( f );
   Author   .PDBASCIIDump ( f );
   RevData  .PDBASCIIDump ( f );
   Supersede.PDBASCIIDump ( f );
@@ -2313,6 +2378,7 @@ char DateCIF[20];
   Source   .MakeCIF ( CIF );
   KeyWords .MakeCIF ( CIF );
   ExpData  .MakeCIF ( CIF );
+  MdlType  .MakeCIF ( CIF );
   Author   .MakeCIF ( CIF );
   RevData  .MakeCIF ( CIF );
   Supersede.MakeCIF ( CIF );
@@ -2340,6 +2406,7 @@ int  i;
     Source   .Copy ( &(TS->Source)    );
     KeyWords .Copy ( &(TS->KeyWords)  );
     ExpData  .Copy ( &(TS->ExpData)   );
+    MdlType  .Copy ( &(TS->MdlType)   );
     Author   .Copy ( &(TS->Author)    );
     RevData  .Copy ( &(TS->RevData)   );
     Supersede.Copy ( &(TS->Supersede) );
@@ -2369,6 +2436,7 @@ int  i;
     Source   .FreeContainer();
     KeyWords .Delete       ();
     ExpData  .FreeContainer();
+    MdlType  .FreeContainer();
     Author   .FreeContainer();
     RevData  .FreeContainer();
     Supersede.FreeContainer();
@@ -2380,14 +2448,17 @@ int  i;
 }
 
 void  CMMDBTitle::TrimInput ( pstr PDBString )  {
-  if (col73)  PDBString[72] = char(0);
+  if (col73)  {
+    if (!strncasecmp(idCode,&(PDBString[72]),4))
+      PDBString[72] = char(0);
+  }
   PadSpaces ( PDBString,80 );
 }
 
 void  CMMDBTitle::write ( RCFile f )  {
 // writes header to PDB binary file
 int  i;
-byte Version=2;
+byte Version=3;
 
   f.WriteByte    ( &Version       );
 
@@ -2404,6 +2475,7 @@ byte Version=2;
   Source   .write ( f );  //  Source
   KeyWords .write ( f );  //  Key words
   ExpData  .write ( f );  //  Experimental data
+  MdlType  .write ( f );  //  Model descriptions
   Author   .write ( f );  //  Author data
   RevData  .write ( f );  //  Revision data
   Supersede.write ( f );  //  Supersede records
@@ -2439,6 +2511,8 @@ byte Version;
   Source   .read ( f );   //  Source
   KeyWords .read ( f );   //  Key words
   ExpData  .read ( f );   //  Experimental data
+  if (Version>2)
+    MdlType.read ( f );   //  Model descriptions
   Author   .read ( f );   //  Author data
   RevData  .read ( f );   //  Revision data
   Supersede.read ( f );   //  Supersede records
