@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    14.06.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +59,7 @@
 //                   ParseResID     ( parses residue ID line         )
 //                   ParseAtomPath  ( parses full atom path          )
 //
-//   (C) E. Krissinel  2000-2010
+//   (C) E. Krissinel  2000-2013
 //
 //  =================================================================
 //
@@ -329,6 +329,15 @@ char N[50];
   }
 }
 
+int CIFGetIntegerD ( int & I, PCMMCIFLoop Loop, cpstr Tag,
+                     int defValue )  {
+int RC,Signal;
+  Signal = 0;
+  RC = CIFGetInteger ( I,Loop,Tag,Signal );
+  if (RC)
+    I = defValue;
+  return RC;
+}
 
 int CIFGetInteger ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                     int & Signal )  {
@@ -1595,8 +1604,9 @@ int ParseAtomPath ( cpstr      ID,
 char    N[100];
 pstr    p,p1;
 int     i,RC;
-Boolean wasRes = False;
+Boolean wasRes;
 
+  wasRes = False;
 
   RC = 0;
 
