@@ -21,7 +21,7 @@
 //
 //  =================================================================
 //
-//    29.01.10   <--  Date of Last Modification.
+//    14.06.13   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +59,7 @@
 //                   ParseResID     ( parses residue ID line          )
 //                   ParseAtomPath  ( parses full atom path           )
 //
-//   (C) E. Krissinel  2000-2010
+//   (C) E. Krissinel  2000-2013
 //
 //  =================================================================
 //
@@ -169,6 +169,8 @@ extern int CIFGetString   ( pstr S, PCMMCIFStruct Struct,
 
 extern int CIFGetInteger  ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                             int & Signal );
+extern int CIFGetIntegerD ( int & I, PCMMCIFLoop Loop, cpstr Tag,
+                            int defValue=MinInt4 );
 extern int CIFGetInteger1 ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                             int nrow );
 
