@@ -5,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or
-//    modify it under the terms of the GNU Lesser General Public
-//    License version 3, modified in accordance with the provisions
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser
-//    General Public License along with this library. If not, copies
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -106,27 +106,51 @@ CXMLObject::~CXMLObject()  {
 }
 
 void  CXMLObject::InitXMLObject()  {
-  objTag   = NULL;
-  objData  = NULL;
-  nObjects = 0;
-  nAlloc   = 0;
-  object   = NULL;
+  parent      = NULL;
+  objTag      = NULL;
+  objData     = NULL;
+  nObjects    = 0;
+  nAlloc      = 0;
+  object      = NULL;
+  nAttributes = 0;
+  nAttrAlloc  = 0;
+  attr_name   = NULL;
+  attr_value  = NULL;
 }
 
 void  CXMLObject::FreeMemory()  {
 int i;
+
   if (objTag)   delete[] objTag;
   if (objData)  delete[] objData;
+
+  objTag   = NULL;
+  objData  = NULL;
+
   if (object)  {
     for (i=0;i<nAlloc;i++)
       if (object[i])  delete object[i];
     delete[] object;
   }
-  objTag   = NULL;
-  objData  = NULL;
+
   nObjects = 0;
   nAlloc   = 0;
   object   = NULL;
+
+  if (attr_name)  {
+    for (i=0;i<nAttrAlloc;i++)  {
+      if (attr_name [i])  delete[] attr_name [i];
+      if (attr_value[i])  delete[] attr_value[i];
+    }
+    FreeVectorMemory ( attr_name ,0 );
+    FreeVectorMemory ( attr_value,0 );
+  }
+
+  nAttributes = 0;
+  nAttrAlloc  = 0;
+  attr_name   = NULL;
+  attr_value  = NULL;
+
 }
 
 void  CXMLObject::SetTag ( cpstr Tag )  {
@@ -175,6 +199,47 @@ int  n;
   *t = char(0);
 }
 
+void  CXMLObject::AddAttribute ( cpstr name, cpstr value )  {
+psvector an,av;
+int      i;
+
+  if (nAttributes>=nAttrAlloc)  {
+    nAttrAlloc = nAttributes + 10;
+    GetVectorMemory ( an,nAttrAlloc,0 );
+    GetVectorMemory ( av,nAttrAlloc,0 );
+    for (i=0;i<nAttrAlloc;i++)  {
+      an[i] = NULL;
+      av[i] = NULL;
+    }
+    for (i=0;i<nAttributes;i++)  {
+      CreateCopy ( an[i],attr_name [i] );
+      CreateCopy ( av[i],attr_value[i] );
+      if (attr_name [i])  delete[] attr_name [i];
+      if (attr_value[i])  delete[] attr_value[i];
+    }
+    FreeVectorMemory ( attr_name ,0 );
+    FreeVectorMemory ( attr_value,0 );
+    attr_name  = an;
+    attr_value = av;
+  }
+
+  CreateCopy ( attr_name [nAttributes],name  );
+  CreateCopy ( attr_value[nAttributes],value );
+  nAttributes++;
+
+}
+
+void  CXMLObject::AddAttribute ( cpstr name, const int iV )  {
+char S[100];
+  sprintf ( S,"%i",iV );
+  AddAttribute ( name,S );
+}
+
+void  CXMLObject::AddAttribute ( cpstr name, const Boolean bV )  {
+  if (bV)  AddAttribute ( name,"Yes" );
+     else  AddAttribute ( name,"No"  );
+}
+
 void  CXMLObject::SetData ( cpstr Data )  {
 pstr p,d;
 int  n;
@@ -220,19 +285,19 @@ pstr d1,d2;
   CreateConcat ( objData,d1,d2 );
 }
 
-void  CXMLObject::SetData ( realtype V, int length )  {
+void  CXMLObject::SetData ( const realtype V, const int length )  {
 char N[500];
   sprintf    ( N,"%-.*g",length,V );
   CreateCopy ( objData,N );
 }
 
-void  CXMLObject::SetData ( int iV, int length )  {
+void  CXMLObject::SetData ( const int iV, const int length )  {
 char N[500];
   sprintf    ( N,"%*i",length,iV );
   CreateCopy ( objData,N );
 }
 
-void  CXMLObject::SetData ( Boolean bV )  {
+void  CXMLObject::SetData ( const Boolean bV )  {
   if (bV)  CreateCopy ( objData,pstr("Yes") );
      else  CreateCopy ( objData,pstr("No")  );
 }
@@ -281,7 +346,7 @@ int         nTags,i,k;
       k++;
     }
   }
-
+  
   if (SName)  {
     if (SName[0]!=char(1))
       AddObject ( XMLObject1 );
@@ -329,7 +394,7 @@ int         nTags,nRows,i,j,k;
     }
     XMLObject1->AddObject ( XMLObject2 );
   }
-
+  
   if (SName)  {
     if (SName[0]!=char(1))
       AddObject ( XMLObject1 );
@@ -463,6 +528,23 @@ pstr        p,p1;
   return XMLObject;
 }
 
+PCXMLObject CXMLObject::GetFirstObject()  {
+  if (nObjects>0)  return object[0];
+  return NULL;
+}
+
+PCXMLObject CXMLObject::GetLastObject()  {
+  if (nObjects>0)  return object[nObjects-1];
+  return NULL;
+}
+
+PCXMLObject CXMLObject::GetObject ( int objectNo )  {
+  if ((0<=objectNo) && (objectNo<nObjects))
+    return object[objectNo];
+  return NULL;
+}
+
+
 void  CXMLObject::AddObject ( PCXMLObject XMLObject, int lenInc )  {
 PPCXMLObject obj1;
 int          i;
@@ -482,6 +564,7 @@ int          i;
 
   if (object[nObjects])  delete object[nObjects];
   object[nObjects] = XMLObject;
+  XMLObject->SetParent ( this );
   nObjects++;
 
 }
@@ -513,6 +596,13 @@ Boolean sngline;
     f.Write ( indstr    );
     f.Write ( pstr("<") );
     f.Write ( objTag    );
+    for (i=0;i<nAttributes;i++)  {
+      f.Write ( " " );
+      f.Write ( attr_name[i] );
+      f.Write ( "=\"" );
+      f.Write ( attr_value[i] );
+      f.Write ( "\"" );
+    }
     if ((!objData) && (nObjects<=0))  {
       f.WriteLine ( pstr("/>") );
       delete[] indstr;
@@ -607,13 +697,12 @@ pstr        S1;
 int         k,k1,k2,rc;
 Boolean     Done;
 
-  k2 = 0; // else may become undefined
-
   FreeMemory();
 
   rc = XMLR_Ok;
 
   k1 = -1;
+  k2 = -1;
   while ((!f.FileEnd()) && (k1<0))  {
     k = strlen(S);
     while ((pos<k) && (k1<0))
@@ -717,15 +806,18 @@ Boolean     Done;
 
   if (rc!=XMLR_Ok)  FreeMemory();
   return rc;
-
+   
 }
 
 
 void  CXMLObject::Copy ( PCXMLObject XMLObject )  {
 int i;
+
   FreeMemory();
+
   CreateCopy ( objTag ,XMLObject->objTag  );
   CreateCopy ( objData,XMLObject->objData );
+
   nObjects = XMLObject->nObjects;
   nAlloc   = nObjects;
   if (nObjects>0)  {
@@ -737,23 +829,45 @@ int i;
       } else
         object[i] = NULL;
   }
+
+  nAttributes = XMLObject->nAttributes;
+  nAttrAlloc  = nAttributes;
+  if (nAttributes>0)  {
+    GetVectorMemory ( attr_name ,nAttrAlloc,0 );
+    GetVectorMemory ( attr_value,nAttrAlloc,0 );
+    for (i=0;i<nAttributes;i++)  {
+      attr_name [i] = NULL;
+      attr_value[i] = NULL;
+      CreateCopy ( attr_name [i],XMLObject->attr_name [i] );
+      CreateCopy ( attr_value[i],XMLObject->attr_value[i] );
+    }
+  }
+
 }
 
 
 void  CXMLObject::write ( RCFile f )  {
 int i;
-  f.CreateWrite ( objTag    );
-  f.CreateWrite ( objData   );
-  f.WriteInt    ( &nObjects );
+  f.CreateWrite ( objTag       );
+  f.CreateWrite ( objData      );
+  f.WriteInt    ( &nObjects    );
   for (i=0;i<nObjects;i++)
     StreamWrite ( f,object[i] );
+  f.WriteInt    ( &nAttributes );
+  for (i=0;i<nAttributes;i++)  {
+    f.CreateWrite ( attr_name [i] );
+    f.CreateWrite ( attr_value[i] );
+  }
 }
 
 void  CXMLObject::read ( RCFile f )  {
 int i;
+
   FreeMemory();
+
   f.CreateRead ( objTag    );
   f.CreateRead ( objData   );
+
   f.ReadInt    ( &nObjects );
   nAlloc = nObjects;
   if (nObjects>0)  {
@@ -763,6 +877,20 @@ int i;
       StreamRead ( f,object[i] );
     }
   }
+
+  f.ReadInt    ( &nAttributes );
+  nAttrAlloc = nAttributes;
+  if (nAttributes>0)  {
+    GetVectorMemory ( attr_name ,nAttrAlloc,0 );
+    GetVectorMemory ( attr_value,nAttrAlloc,0 );
+    for (i=0;i<nAttributes;i++)  {
+      attr_name [i] = NULL;
+      attr_value[i] = NULL;
+      f.CreateRead ( attr_name [i] );
+      f.CreateRead ( attr_value[i] );
+    }
+  }
+
 }
 
 
